/////////////////////////////////////////////////////////////////////////////////////////////
//	See PO_Assert.h for docs
/////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////
//	INCLUDES
/////////////////////////////////////////////////////////////////////////////////////////////

#include "PO_Assert.h"

#if PO_ASSERT_REPORT==PO_ASSERT_REPORT_OFF || defined(NDEBUG)
	
	// nothing, its off
	
#else


	/////////////////////////////////////////////////////////////////////////////////////////////
	//	PRIVATE UTILS
	/////////////////////////////////////////////////////////////////////////////////////////////


	static unsigned char*
	__mb_strcpy(unsigned char* dest, const char* source)
	{
		Boolean		moreQ;
		
		do{
			moreQ = (*dest++ = *source++);
		}while(moreQ);
		
		return --dest;
	}

	static const char*
	__mb_numtostr(int num)
	{
		static Str31 buf;
		NumToString((long) num, buf);
		buf[ buf[0]+1 ] = '\0';
		return (char*) &buf[1];
	}
	
	static long
	__strlen(const char* inS)
	{
		long		result = 0;
		while(*inS){
			result++;
			inS++;
		}
		
		return result;
	}

	static void GenerateMessage(const char* inAssert,const char* inFile,long inLineNum,Str255 outMessage)
	{
		unsigned char*	s				= &outMessage[1];
		
		s = __mb_strcpy(s, "Assertion failed: ");
		s = __mb_strcpy(s, inAssert);
		s = __mb_strcpy(s, ", file ");
		s = __mb_strcpy(s, inFile);
		s = __mb_strcpy(s, ", line ");
		s = __mb_strcpy(s, __mb_numtostr(inLineNum));
		
		outMessage[0] = s - outMessage - 1;
	}


	/////////////////////////////////////////////////////////////////////////////////////////////
	//	IMPLEMENTATION
	/////////////////////////////////////////////////////////////////////////////////////////////

	void __po_assert(const char* inAssert, const char* inFile, long inLine,bool inFatalQ)
	{
		unsigned char		message[500];
		
		GenerateMessage(inAssert,inFile,inLine,message);
		
		PO_Assert_Report(message,inFatalQ);
	}
	
		#include "PO_DebugUtils.h"
	
	void	__po_assert2(const char* inAssert,const char* inFile, long inLine,long inValue,bool inFatal)
	{
		unsigned char		message[500];
		
		PO_DebugUtils::FormatCStr((char*)message+1,"Assertion failed: %s, value: %x, file %s, line %l",
									inAssert,inValue,inFile,inLine);
		message[0] = __strlen((char*)message+1);
		
		PO_Assert_Report(message,inFatal);
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////////
	//	MACSBUG REPORTER
	/////////////////////////////////////////////////////////////////////////////////////////////

	#if PO_ASSERT_REPORT == PO_ASSERT_REPORT_MACSBUG

		
		void	PO_Assert_Report(ConstStr255Param inMessage,bool inFatalQ)
		{
			StringPtr	s = const_cast<StringPtr>(inMessage);
			
			if(!inFatalQ){
				s[++s[0]] = ';';
				s[++s[0]] = 'g';
			}
			
			PO_DebugUtils::DebugStr(s);
		}

	#elif PO_ASSERT_REPORT == PO_ASSERT_REPORT_COUT
		
		#include <iostream>
		#include <stdlib.h>
		
		void	PO_Assert_Report(ConstStr255Param inMessage,bool inFatalQ)
		{
			StringPtr	s = const_cast<StringPtr>(inMessage);
			
			s[s[0]+1] = 0;
			
			cout << reinterpret_cast<char*>(s + 1) << endl;
			
			if(inFatalQ){
				exit(0);
			}	
		}
		
	#endif
#endif

