#include "PO_ObjectPool.h"
#include "PO_Assert.h"
#include <string.h>


static QHdr  sPoolList = {0,NULL,NULL};


void*		PO_ObjectPool::AllocateObject()
{
	QElemPtr		next = NULL;
	Boolean			gotItQ;
	
	// this strange looking code is a thread safe way to dequeue the first elem
	// of a queue
	do{
		next = fFreeObjects.qHead;
		if(next != NULL){
			gotItQ = (Dequeue(next,&fFreeObjects) == noErr);
		}else{
			gotItQ = true;	// got null
		}
	}while( !gotItQ );
	
	if(next != NULL){
		fNumFree--;
	}
	
	// if we are growable,
	// check to see if we are running low
	
	if(fGrowSize != 0){
		if(GetNumFree() < fMinFreeSize){
			fPeakedQ = true;
		}
	}
	
	// calculate highest usage
	
	if(fTotalObjects - fNumFree > fHighCount){
		fHighCount = fTotalObjects - fNumFree;
	}
	
	return next;

}


void		PO_ObjectPool::FreeObject(void* object)
{
	Enqueue((QElemPtr)object,&fFreeObjects);
	fNumFree++;
}


void*	PO_ObjectPool::AllocateRaw(UInt32 size)
{
	return new char[size];
}

void	PO_ObjectPool::FreeRaw(void* p)
{
	delete [] ((char*)p);
}



PO_ObjectPool::~PO_ObjectPool()
{
	::Dequeue((QElemPtr)&fPoolElem,&sPoolList);
	
	assert(fTotalObjects == fNumFree);
	
	QElemPtr		p = fElems.qHead;
	while(p != NULL){
		QElemPtr		next = p->qLink;
		FreeRaw((void*)p);
		p = next;
	}
}


OSErr		PO_ObjectPool::IncreasePoolSize(UInt32 numObjects)
{
	OSErr		err = noErr;
	UInt32		elemSize = sizeof(ObjectElem) + (numObjects * fObjectSize);
	ObjectElem*	elem = (ObjectElem*)AllocateRaw(elemSize);
	
	if(elem != NULL){
		memset(elem,0,elemSize);
		elem->numObjects = numObjects;
		
		Enqueue((QElemPtr)elem,&fElems);
		
		Ptr		p = elem->store;
		for(UInt32 z = 0;z<numObjects;z++){
			Enqueue((QElemPtr)p,&fFreeObjects);
			p += fObjectSize;
		}
		
		fNumFree += numObjects;
		fTotalObjects += numObjects;
	}else{
		err = -108;
	}
	
	
	
	return err;
}


PO_ObjectPool::PO_ObjectPool(UInt32 objectSize)
{
	CommonInit(objectSize);
}

void	PO_ObjectPool::Idle()
{
	PoolElem*		thisElem = (PoolElem*)sPoolList.qHead;
	while(thisElem != NULL){
		thisElem->thisObj->IdleSelf();
		thisElem = (PoolElem*)thisElem->qLink;
	}
}


void			PO_ObjectPool::IdleSelf()
{
	if(fPeakedQ){
		UInt32		curSize = GetPoolSize();
		UInt32		sizeToIncrease = fGrowSize;
		
		if(curSize == 0){
			sizeToIncrease = fInitialSize;
		}
		
		if(curSize + sizeToIncrease > fMaxSize){
			sizeToIncrease = fMaxSize - curSize;
		}
		
		if(sizeToIncrease > 0){
			OSErr	err = IncreasePoolSize(sizeToIncrease);
			if(err != noErr){
				if(fProc != NULL){
					(*fProc)(this,fUserData);
				}else{
					assert(err == noErr);
				}
			}
		}
		
		fPeakedQ = false;
	}
}


void			PO_ObjectPool::CommonInit(UInt32 inObjectSize)
{
	fPoolElem.qLink = NULL;
	fPoolElem.thisObj = this;
	::Enqueue((QElemPtr)&fPoolElem,&sPoolList);

	fHighCount = 0;
	fGrowSize = 0;
	fPeakedQ = false;
	fObjectSize = inObjectSize;
	fNumFree = 0;
	fTotalObjects = 0;

	if(fObjectSize < 6){	// QElem size
		fObjectSize = 6;
	}
	
	fElems.qHead = fElems.qTail = NULL;
	fFreeObjects.qHead = fFreeObjects.qTail = NULL;
}


PO_ObjectPool::PO_ObjectPool(UInt32 objectSize,UInt32 inInitialSize,UInt32 inMinFreeSize,
							UInt32 inGrowSize,UInt32 inMaxSize,CantGrowProc inProc ,
							void* inUserData )
{
	CommonInit(objectSize);
	
	fInitialSize = inInitialSize;
	fMinFreeSize = inMinFreeSize;
	fGrowSize = inGrowSize;
	fMaxSize = inMaxSize;
	fProc = inProc;
	fUserData = inUserData;
	fPeakedQ = true;

	IdleSelf();
}

UInt32			PO_ObjectPool::GetActualFree()
{
	UInt32			result = 0;
	
	QElemPtr		p = fFreeObjects.qHead;
	while(p != NULL){
		result++;
		p = p->qLink;
	}
	
	return result;
	
}




