#pragma once
#ifndef INCLUDED_PO_OBJECTPOOL
#define INCLUDED_PO_OBJECTPOOL

////////////////////////////////////////////////////////
//
//	PO_ObjectPool
//
//
//	This class allows you create either a static sized or
// 	dynamic sized pool of object storage.
//
//	allocation and deallocation of objects FAST and interrupt
//	save
//
//	You probably would want to use the template based version
//	in PO_TObjectPool.h unless to need raw storage for buffers
//	and the like.
//
////////////////////////////////////////////////////////



class	PO_ObjectPool
{
public:
	typedef pascal void	(*CantGrowProc)(PO_ObjectPool* thisPool,void* userData);

	// constructs fixed sized pool
	
	PO_ObjectPool(UInt32 objectSize);
	
	// constructs growable pool
	
	PO_ObjectPool(UInt32 objectSize,UInt32 inInitialSize,UInt32 inMinFreeSize,
							UInt32 inGrowSize,UInt32 inMaxSize,CantGrowProc inProc = NULL,
							void* inUserData = NULL);
	
	
	// call at safe time to allow growable pools to grow.
	
	static 	void	Idle();

	// manually grows pool
	
	OSErr			IncreasePoolSize(UInt32 inNumObjects);

	
	// accessors
	
	UInt32			GetNumFree()							{ return fNumFree;      }
	UInt32			GetObjectSize()							{ return fObjectSize;   }
	UInt32			GetPoolSize()							{ return fTotalObjects; }
	UInt32			GetActualFree();	// for debugging
	UInt32			GetHighestUsage()						{ return fHighCount; }


	// allocators and deallocators

	void*			AllocateObject();
	void			FreeObject(void* object);




	virtual 	~PO_ObjectPool();
protected:
	void			IdleSelf();
	void			CommonInit(UInt32 inObjectSize);
	virtual void*	AllocateRaw(UInt32 size);
	virtual void	FreeRaw(void* p);
	

	struct ObjectElem
	{
		QElemPtr		qLink;
		short			qType;
		
		UInt32			numObjects;
		char			store[2];
	};
	
	QHdr		fElems;
	QHdr		fFreeObjects;
	UInt32		fObjectSize;
	UInt32		fNumFree;
	UInt32		fTotalObjects;

	// for growable pool
	UInt32			fInitialSize;
	UInt32			fMinFreeSize;
	UInt32			fGrowSize;
	UInt32			fMaxSize;
	bool			fPeakedQ;
	CantGrowProc	fProc;
	void*			fUserData;
	UInt32			fHighCount;
	
	struct PoolElem
	{
		QElemPtr	qLink;
		short		qType;
		
		PO_ObjectPool*	thisObj;
	};
	
	PoolElem		fPoolElem;
		
};





#endif
