#pragma once
#ifndef INCLUDED_PO_TOBJECTPOOL
#define INCLUDED_PO_TOBJECTPOOL

////////////////////////////////////////////////////////
//
//	PO_TObjectPool
//
//	This is a template version of the PO_ObjectPool class
//
//	This class allows you create either a static sized or
// 	dynamic sized pool of object storage.
//
//	allocation and deallocation of objects FAST and interrupt
//	save
//
//	See below for example usage
//
////////////////////////////////////////////////////////



#include "PO_ObjectPool.h"


template <class T>
class PO_TObjectPool : public PO_ObjectPool
{
public:
	
	// non growing pool (Use PO_ObjectPool::IncreasePoolSize) to set the size
	
	PO_TObjectPool()	:	PO_ObjectPool(sizeof(T) + sizeof(PO_ObjectPool*)) {}
	
	// growable pool
	
	PO_TObjectPool(UInt32 inInitialSize,UInt32 inMinFree,UInt32 inGrowSize,UInt32 inMaxSize)
					:	PO_ObjectPool(sizeof(T) + sizeof(PO_ObjectPool*),inInitialSize,
									inMinFree,inGrowSize,inMaxSize) {}
	
	// ctors and dtors are not called.
	// use the supplied operator new and Tpool_delete functions
	
			T*		New();
	static	void	Delete(T* object);

};


template <class T>
T*	PO_TObjectPool<T>::New()
{
	PO_ObjectPool**		result = (PO_ObjectPool**)AllocateObject();
	
	if(result != NULL){
		*result = this;
		result++;
	}
	
	return (T*) result;
}



template <class T>
void	PO_TObjectPool<T>::Delete(T* object)
{
	PO_TObjectPool<T>**	pool = (PO_TObjectPool<T>**) object;
	
	pool--;
	
	(*pool)->FreeObject(pool);		
}




////////////////////////////////////////////////////////////
//
//	Use these functions to have any normal class
//  use the a pool
//
///////////////////////////////////////////////////////////


template <class T>
void* operator new(UInt32 inSize,PO_TObjectPool<T>& inPool) throw()
{
	if(inSize == sizeof(T)){
		return inPool.New();
	}else{
		return NULL;
	}
}


template <class T>
void	Tpool_delete(T* inObj)
{
	if(inObj != NULL){
		inObj->~T();
		PO_TObjectPool<T>::Delete(inObj);
	}
}




//////////////////////////////////////////////////////////////////////////////
//
//	Example:
//
//	Say you have a class called "Bobo" and you would like to create a pool
// 	of these objects for fast allocation or for use at interrupt.
//

#if 0

	// construct a standard empty pool of "Bobos"
	
	PO_TObjectPool<Bobo>		myPool;
	
	// increase the size of the pool by 10 (making it hold 10 since it was empty to start with)
	
	OSStatus	err = myPool.IncreasePoolSize(10);
	if(err == noErr){
	
		// construct an object from the pool
		
		Bobo*		myBobo = new(myPool) Bobo;

		// destruct it and return it's storage to the pool

		Tpool_delete(myBobo);
	}
	
#endif


//	Now if you want, you can construct a pool that will automatically grow
//	based on the critieria you specify

#if 0

	// construct a "growable" pool
	
	const UInt32		kInitialPoolSize = 10;	// start with room for 10
	const UInt32		kMinFreeSize = 3;		// don't allow pool to go below 3 free
	const UInt32		kGrowSize = 5;			// grow by 5 each time the pool fills up
	const UInt32		kMaxSize = 30;			// but don't get any bigger than 30
	
	PO_TObjectPool<Bobo>		myPool(kInitialPoolSize,kMinFreeSize,kGrowSize,kMaxSize);
	
	do{
		DoStuff(myPool);		// uses objects in the pool
	
		// give time to the pool to allow it to grow
		PO_ObjectPool::Idle();
		
	}while(!gDone);
	
#endif
//
//	
//
//////////////////////////////////////////////////////////////////////////////





#endif

