#include "PO_Queue.h"


PO_Queue::PO_Queue()
{
	_iters.qHead = 
	_iters.qTail = 
	_queue.qHead = 
	_queue.qTail = NULL;
	_numElems = 0;
}


PO_Queue::~PO_Queue()
{
	if(_iters.qHead != NULL){
		FirstIter()->QueueDestroyed();
	}
}

OSErr	PO_Queue::Dequeue(PO_MQueueElem*		elem)
{	
	if(::Dequeue((QElemPtr)&elem->_elem,&_queue) == noErr){
		_numElems--;
		if(_iters.qHead != NULL){
			PO_QIterator*		iter = FirstIter();
			iter->ElemDequeued(elem);
		}
		return noErr;
	}else{
		return paramErr;
	}
}


void	PO_Queue::DequeueAll()
{
	_queue.qHead = _queue.qTail = NULL;
	_numElems = 0;
	
	if(_iters.qHead != NULL){
		FirstIter()->QueueEmpty();
	}
}



PO_MQueueElem::PO_MQueueElem()
{
	_elem.qLink = NULL;
	_elem.qObject = this;
}

PO_MQueueElem*		PO_MQueueElem::Next()
{
	if(_elem.qLink == NULL){
		return NULL;
	}else{
		return ((Elem*)_elem.qLink)->qObject;
	}
}



PO_QIterator::PO_QIterator()
{
	Init();
}

PO_QIterator::~PO_QIterator()
{
	if(_queue != NULL){
		_queue->RemoveIterator(this);
	}
}


PO_QIterator::PO_QIterator(PO_Queue&	queue)
{
	Init();

	SetQueue(&queue);
}


void			PO_QIterator::SetQueue(PO_Queue*	queue)
{
	if(_queue != NULL){
		_queue->RemoveIterator(this);
	}
	_queue = queue;
	if(_queue != NULL){
		_queue->AddIterator(this);
		
		_curElem = _queue->GetHead();
	}
}


void			PO_QIterator::ElemDequeued(PO_MQueueElem* elem)
{
	_listModifyCount++;
	
	if(elem == _curElem){
		_curElem = elem->Next();
	}
	
	PO_QIterator*		next = (PO_QIterator*)Next();
	if(next != NULL){
		next->ElemDequeued(elem);
	}
}


void			PO_QIterator::QueueDestroyed()
{
	Init();
}


void			PO_QIterator::Init()
{
	_curElem = NULL;
	_queue = NULL;
	_listModifyCount = 0;
}

/*
#include <stddef.h>

UInt32			t = offsetof(PO_QIterator, _curElem);
*/

PO_MQueueElem*		PO_QIterator::GetNext()
{	
tryAgain:

	UInt32				saveModifyCount = _listModifyCount;
	
	PO_MQueueElem*			result = _curElem;
	
	if(result != NULL){
		_curElem = result->Next();
	}
	
	if(saveModifyCount != _listModifyCount){
		Reset();	// somebody monkeyed with the list
					// we have to start over from the begining.
		goto tryAgain;
	}
	
	return result;
}


void			PO_QIterator::QueueEmpty()
{
	_curElem = NULL;
}


void			PO_QIterator::ElemQueued(PO_MQueueElem* elem)
{
	_listModifyCount++;

	if(elem->Next() == _curElem){
		_curElem = elem;
	}
	
	PO_QIterator*		next = (PO_QIterator*)Next();
	if(next != NULL){
		next->ElemQueued(elem);
	}
}

void			PO_QIterator::Reset()
{
	_curElem = _queue->GetHead();
}











void	PO_Queue::Enqueue(PO_MQueueElem*		elem)
{
	::Enqueue((QElemPtr)&elem->_elem,&_queue);
	_numElems++;
	
	if(_iters.qHead != NULL){
		FirstIter()->ElemQueued(elem);
	}
}

void	PO_Queue::AddIterator(PO_QIterator* iter)
{
	::Enqueue((QElemPtr)&iter->_elem,&_iters);
}


void	PO_Queue::RemoveIterator(PO_QIterator* iter)
{
	::Dequeue((QElemPtr)&iter->_elem,&_iters);
}

PO_MQueueElem*			PO_Queue::GetHead()
{
	PO_MQueueElem*			result = NULL;
	
	if(_queue.qHead != NULL){
		result = (PO_MQueueElem*)		((PO_MQueueElem::Elem*)_queue.qHead)->qObject;
	}
	
	return result;
}

PO_MQueueElem*			PO_Queue::GetTail()
{
	PO_MQueueElem*		result = NULL;
	
	if(_queue.qTail != NULL){
		result = (PO_MQueueElem*)		((PO_MQueueElem::Elem*)_queue.qTail)->qObject;
	}

	return result;
}

