#pragma once
#ifndef INCLUDED_PO_QUEUE
#define INCLUDED_PO_QUEUE

//////////////////////////////////////////////////////
//
//	PO_Queue
//	Base implementation for a queue of objects
//	NOTE: You probably want to use the template versions
//		  found in PO_TQueue.h
//
//////////////////////////////////////////////////////




class PO_QIterator;



//////////////////////////////////////////////////////
//
//	PO_MQueueElem
//	Mix in queue element class
//
//////////////////////////////////////////////////////

class PO_MQueueElem
{
	friend class PO_Queue;
public:
	PO_MQueueElem();
	virtual ~PO_MQueueElem() {}
	
	PO_MQueueElem*		Next();
	
protected:
	
	struct Elem {
		QElemPtr		qLink;
		short			qType;
		PO_MQueueElem*	qObject;
	};
	
	Elem	_elem;
	
};

//////////////////////////////////////////////////////
//
//	PO_Queue
//	base queue class
//
//////////////////////////////////////////////////////

class PO_Queue
{
	friend class PO_QIterator;
public:
	PO_Queue();
	virtual ~PO_Queue();
	
	UInt32	GetNumElems()				{return _numElems;}
	UInt32	Size()						{return _numElems;}
	void	Enqueue(PO_MQueueElem*		elem);
	OSErr	Dequeue(PO_MQueueElem*		elem);
	
	void	DequeueAll(); 
	
	PO_MQueueElem*			GetHead();
	PO_MQueueElem*			GetTail();
	
protected: 
	
	UInt32		_numElems;

	PO_QIterator*	FirstIter()			{return (PO_QIterator*)((PO_MQueueElem::Elem*)_iters.qHead)->qObject;}
	void	RemoveIterator(PO_QIterator* iter);
	void	AddIterator(PO_QIterator* iter);
	
	QHdr			_queue;
	QHdr			_iters;
};



//////////////////////////////////////////////////////
//
//	PO_QIterator
//	base iterator class
//
//////////////////////////////////////////////////////

class PO_QIterator : public PO_MQueueElem
{
	friend class PO_Queue;
	
public:
	PO_QIterator();
	PO_QIterator(PO_Queue&	queue);
	virtual ~PO_QIterator();
	
	void			SetQueue(PO_Queue*	queue);

	Boolean			MoreQ()		{return _curElem != NULL;}
	PO_MQueueElem*	GetNext();
	void			Reset();
	
protected:
	void			ElemQueued(PO_MQueueElem* elem);
	void			ElemDequeued(PO_MQueueElem* elem);
	void			QueueEmpty();
	void			QueueDestroyed();
protected:
	void			Init();
	
	PO_Queue*		_queue;
	PO_MQueueElem*	_curElem;
	short			_listModifyCount;
};



#endif
