#pragma once
#ifndef INCLUDED_PO_STREAM
#define INCLUDED_PO_STREAM

////////////////////////////////////////////////////////////////////////////////////////
//
//	INTERFACE:		PO_Stream
//
//	DESCRIPTION:	Abstract base class for standard stream implementation
//					More clients will want to use the PO_StreamFactory and
//					PO_TagStream objects to manipulate streams
//
//	AUTHORS:		Bill Hubauer (BH)
//
//	HISTORY:		See implementation file for history
//
////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////
//	INCLUDES
////////////////////////////////////////////////////////////////////////////////////////


#ifndef __TYPES__
#include <types.h>
#endif

#ifndef INCLUDED_PO_REFCOUNTED
#include "PO_RefCounted.h"
#endif

#ifndef INCLUDED_PO_STREAMREF
#include "PO_StreamRef.h"
#endif


////////////////////////////////////////////////////////////////////////////////////////
//	PULIC INTERFACE
////////////////////////////////////////////////////////////////////////////////////////
class PO_StreamEventHandler;
template <class T>	class PO_THandleList;

class PO_Stream : public PO_RefCounted
{
public:
	
	PO_Stream(Boolean writableQ = true) 
		: fWritableQ(writableQ),fDirtyQ(false),fHandlerList(NULL) {}
	virtual ~PO_Stream();
	
	virtual OSStatus	Status()						{ return noErr; }
	
	virtual	UInt32		Size() const = 0;
	virtual UInt32		Position() const = 0;
	
	virtual Boolean		WritableQ() const			{ return fWritableQ; }
	virtual Boolean		DirtyQ() const				{ return fDirtyQ; }
	virtual void		ClearDirtyFlag();

	virtual OSErr		Resize(UInt32 inNewSize) = 0;
	virtual OSErr		Seek(UInt32 inNewPosition) = 0;
	virtual OSErr		Read(void* inBuffer,UInt32 inBytesToRead) = 0;
	virtual OSErr		Write(const void* inBuffer,UInt32 inBytesToWrite) = 0;
	
	
	virtual OSErr		CopyTo(PO_StreamRef inDest,UInt32 inSize);	// copies from cur position
			OSErr		Peek(UInt32 inPos,void* inBuffer,UInt32 inBytesToPeek);
			OSErr		Poke(UInt32	inPos,const void* inBuffer,UInt32 inBytesToPoke);

			OSStatus	AttachEventHandler(PO_StreamEventHandler* inHandler);
			void		RemoveEventHandler(PO_StreamEventHandler* inHandler);
			void		BroadcastStreamMessage(UInt32 inMessage,void* inParam);

protected:
 
			void		SetWritableQ(Boolean writableQ)	{fWritableQ = writableQ;}
			void		SetDirtyQ()	{ fDirtyQ = true; }

private:
	
	Boolean				fWritableQ;
	Boolean				fDirtyQ;
	
	PO_THandleList<PO_StreamEventHandler*>*		fHandlerList;

};



#endif
