#ifndef _PO_DebugUtils_
	#include "PO_DebugUtils.h"
#endif

#ifndef __MACMEMORY__
#include <MacMemory.h>
#endif

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif

static const UInt32 kModernMachine			= 0x02000;
static const SInt8* kMacsBugActiveModern	= reinterpret_cast<SInt8*>(0xBFF);
static const SInt8* kMacsBugActiveOld		= reinterpret_cast<SInt8*>(0x120);
static const UInt32* kMacsBugInstalled		= reinterpret_cast<UInt32*>(0x120);

// private function
static char* CopyCStr(char* destStr, const char* str);
static char* CopyNumStr(char* destStr, const long num);
static char* LocalFormatCStr(const char** formatStr, ...);



void PO_DebugUtils::Debugger()
{
	if (MacsBugInstalledQ())
		::Debugger();
}

void PO_DebugUtils::DebugStr(ConstStr255Param str)
{
	if (MacsBugInstalledQ())
		::DebugStr(str);
}

Boolean PO_DebugUtils::MacsBugInstalledQ()
{
	return ((*kMacsBugInstalled == 0) || (*kMacsBugInstalled == 0xFFFFFFFF)) ? false : true;
}

Boolean PO_DebugUtils::MacsBugActiveQ()
{
	UInt32			sysZone = static_cast<UInt32>(SystemZone());
	SInt8			active;
	
	if (sysZone >= kModernMachine)
		active = *kMacsBugActiveModern;
	else
		active = *kMacsBugActiveOld;
	
	return active < 0 ? true : false;
}


char* PO_DebugUtils::FormatCStr(char *outputStr, const char* formatStr, ...)
{
	CopyCStr(outputStr, LocalFormatCStr(&formatStr));
	return outputStr;
}


void PO_DebugUtils::DebugCStr(const char* formatStr, ...)
{
	char pStr[256];
	pStr[0] = CopyCStr(&pStr[1], LocalFormatCStr(&formatStr)) - &pStr[1];
	PO_DebugUtils::DebugStr((StringPtr)pStr);
}



//////////////// util functions for DebugCStr and FormatCStr

#pragma warn_possunwant off
// Copy the str to the destStr and return a pointer to the null-terminating byte.
static char* CopyCStr(char* destStr, const char* str)
{
	while (*destStr++ = *str++)
		continue;
	return destStr-1;
}
#pragma warn_possunwant reset


static char* CopyPStr(char* destStr,ConstStr255Param inP)
{
	BlockMoveData(inP+1,destStr,inP[0]);
	destStr += inP[0];
	*destStr = 0;
	
	return destStr;
}

// Copy the number to the destStr and return a pointer to the null-terminating byte.
static char* CopyNumStr(char* destStr, const long num)
{
	NumToString(num, (StringPtr)destStr);
	destStr[*destStr+1] = '\0';
	return CopyCStr(destStr, destStr+1);
}


// Format the string according to the specifiers are return a pointer to a static output string.
// The value in the output string is not preserved across multiple calls to this function.
// The format specifiers are: %s == string, %d == short, %l == long, %x = long hex.
static char* LocalFormatCStr(const char** formatStr, ...)
{
	char** param = (char**)&formatStr[0] + 1;
	static char outputStr[500];
	char* outputPtr = outputStr;
	
	for (short i=0; formatStr[0][i]; i++)
	{
		// Perform the formatting based on the next character
		if (formatStr[0][i] == '%')
		{
			i += 1;
			
			// %s is string
			if (formatStr[0][i] == 's')
				outputPtr = CopyCStr(outputPtr, *param++);
			
			// %P is pascal string
			else if(formatStr[0][i] == 'P')
				outputPtr = CopyPStr(outputPtr,*((StringPtr*)param)++);
				
			// %d is short decimal
			else if (formatStr[0][i] == 'd')
				outputPtr = CopyNumStr(outputPtr, *((short*)param)++);
			
			// %l is long decimal
			else if (formatStr[0][i] == 'l')
				outputPtr = CopyNumStr(outputPtr, *((long*)param)++);
			
			// %x is long hexidecimal
			else if (formatStr[0][i] == 'x')
			{
				outputPtr = CopyCStr(outputPtr, "0x");

				long number=*((long*)param)++;
				for (short j=7; j>=0; j--)
				{
					char digit = number & 0xF;
					if (digit < 10)
						outputPtr[j] = digit + '0';
					else
						outputPtr[j] = digit - 10 + 'A';
						
					number >>= 4;
				}
				outputPtr += 8;
			}
			
			// %p is Point
			else if (formatStr[0][i] == 'p')
			{
				Point* point = *((Point**)param)++;
				outputPtr = CopyCStr(outputPtr, "(h=");
				outputPtr = CopyNumStr(outputPtr, point->h);
				outputPtr = CopyCStr(outputPtr, ", v=");
				outputPtr = CopyNumStr(outputPtr, point->v);
				outputPtr = CopyCStr(outputPtr, ")");
			}
			
			// %r is Rect
			else if (formatStr[0][i] == 'r')
			{
				Rect* rect = *((Rect**)param)++;
				outputPtr = CopyCStr(outputPtr, "(l=");
				outputPtr = CopyNumStr(outputPtr, rect->left);
				outputPtr = CopyCStr(outputPtr, ", t=");
				outputPtr = CopyNumStr(outputPtr, rect->top);
				outputPtr = CopyCStr(outputPtr, ", r=");
				outputPtr = CopyNumStr(outputPtr, rect->right);
				outputPtr = CopyCStr(outputPtr, ", b=");
				outputPtr = CopyNumStr(outputPtr, rect->bottom);
				outputPtr = CopyCStr(outputPtr, ")");
			}
			
			// %o is OSType
			else if (formatStr[0][i] == 'o')
			{
				*outputPtr++ = *((char*)param)++;
				*outputPtr++ = *((char*)param)++;
				*outputPtr++ = *((char*)param)++;
				*outputPtr++ = *((char*)param)++;
				*outputPtr = '\0';
			}
			
			else
				DebugStr("\pAssert format error");
		}
		
		else
			*outputPtr++ = formatStr[0][i];
	}
	
	// Null terminate and then return the static output string.
	*outputPtr = 0;
	return outputStr;
}


