#include "TR_DeferredActions.h"
#include "PO_Assert.h"


TR_DeferredActions::TR_DeferredActions
(
	UInt32	inInitialElems,			// initial number of elements
	UInt32	inMinNumFree,			// number of free that must be maintained
									// after which the inGrowSize is allocated
	UInt32	inGrowSize,				// grow increment if minNumFree is exceded
	UInt32	inMaxSize				// max number of elements to grow to.
)
	:	fElemPool(inInitialElems,inMinNumFree,inGrowSize,inMaxSize)
{

}

		
TR_DeferredActions::~TR_DeferredActions()
{
	assert(fQueue.GetNumElems() == 0);
	PO_TQIterator<ActionElem>		iter(fQueue);
	while(iter.MoreQ()){
		ActionElem*		elem = iter.GetNext();
		fQueue.Dequeue(elem);
		Tpool_delete(elem);
	}
}


		
void	TR_DeferredActions::Idle()
{

	// run actions
	
	PO_TQIterator<ActionElem>		iter(fQueue);
	while(iter.MoreQ()){
		ActionElem*		thisElem = iter.GetNext();
		
		ActionProc		theProc = thisElem->fProc;
		void*			param1 = thisElem->fParam1;
		void*			param2 = thisElem->fParam2;
	
		fQueue.Dequeue(thisElem);
		Tpool_delete(thisElem);
		
		(*theProc)(param1,param2);
		
	}
	
	// in case no one else is idling the pools, I will do it here.
	
	fElemPool.Idle();
}

		
bool	TR_DeferredActions::DeferAction(ActionProc inProc,void* param1,void* param2)
{
	bool		result = false;
	
	ActionElem*	newElem = new(fElemPool) ActionElem;
	if(newElem != NULL){
		newElem->fProc = inProc;
		newElem->fParam1 = param1;
		newElem->fParam2 = param2;
		fQueue.Enqueue(newElem);
		result = true;
	}
	
	return result;
}



