#pragma once
#ifndef INCLUDED_TR_REQUESTOR_ATK
#define INCLUDED_TR_REQUESTOR_ATK
//////////////////////////////////////////////////////////////////////////
//	
//	TR_OutgoingRequest_ATK
//
//	This class defines the AppleTalk specific implementation
//	of a request object
//
//	THIS IS NOT A PUBLIC INTERFACE FILE
//
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
//	INCLUDES
//////////////////////////////////////////////////////////////////////////

#include "TR_Request.h"
#include "PO_Semaphore.h"

//////////////////////////////////////////////////////////////////////////
//	IMPLEMENTATION SPECIFIC CLASSES
//////////////////////////////////////////////////////////////////////////

class TR_OutgoingRequest_ATK : public TR_OutgoingRequest
{
	public:
	
		typedef UInt32		Priority;
		enum{
			kLowPriority,
			kNormalPriority,
			kHighPriority
		};
		
		TR_OutgoingRequest_ATK
		(
 			const 	TR_NetworkAddress& 	inAddress,
			PO_Ref<TR_Buffer>			inReqData,
			PO_Ref<TR_Buffer>			inRespBuffer,
			Priority					inPriority = kNormalPriority
		);
		
		
		// async operation
		
		typedef pascal void (*NotifyProc)(void* userData,TR_OutgoingRequest* inRequest);
		void		NotifyWhenComplete(NotifyProc inProc,void* inUserData);	

		// sync operation
		
		void		WaitUntilComplete();
		
		// polling
		
		Boolean		IsCompleteQ();
		
		// caneling
		
		void		Cancel();		// not interrupt safe
		
		// results query
		
		OSStatus			GetResult();
		PO_Ref<TR_Buffer>	GetResponse();
	
	
	
	
	
	
	
		// private
		static void			ATPCompletionStub(ATPParamBlock* inPB);
	private:
		OSStatus			fResult;
		PO_Ref<TR_Buffer>	fRespBuffer;
		PO_Ref<TR_Buffer>	fReqData;
		Priority			fPriority;
		BDSType				fBDS;
		
		struct ATPExtPB : ATPParamBlock
		{
			long					globals;
			TR_OutgoingRequest_ATK*	thisObj;
		};
		
		ATPExtPB			fPB;
		NotifyProc			fNotifyProc;
		void*				fNotifyUserData;
		bool				fSettingNotifyQ;
		PO_Semaphore		fNotifyProcCalled;
		

		void				ATPCompletion();
		void				DoNotify();


	private:
		friend class	TR_RequestFactory;
		void*	operator new(UInt32 inSize,TR_Allocator* inAllocator);
		void	operator delete(void* inObj);
		TR_Allocator*	fAllocator;

};

#endif

