////////////////////////////////////////////////////////////////
//
//	CLASS:			TR_Responder
//	
//	DESCRIPTION:	TR_Responder is the public interface
//					for receiving and responding to requests
//
//	AUTHORS:		Bill Hubauer (BH)
//
//	HISTORY:
//		WHEN		WHO		WHAT
//		----		---		----
//		5/22		BH		Created and Debugged
//
////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////
//	INCLUDES
////////////////////////////////////////////////////////////////

#include "TR_Buffer.h"
#include "TR_Allocator.h"
#include <new>
#include "PO_Assert.h"
#include "PO_Globals.h"
#include "TR_Responder_ATK.h"



////////////////////////////////////////////////////////////////
//	IMPLEMENTATION
////////////////////////////////////////////////////////////////


OSStatus	TR_ResponderFactory::New
(
	const TR_NetworkName&			 	inName,
	UInt32 								inInitialNumRequests,
	UInt32								inMinFreeRequests,
	UInt32								inGrowSize,
	UInt32								inMaxNumberRequests,
	
	UInt32								inMaxRequestBufferSize,
	TR_Allocator*						inRequestAllocator,
	TR_Responder*&						outResponder
)
{
	outResponder = NULL;
	OSStatus		err = noErr;
	
	if(inName.GetType() == kNameTypeAppleTalk){
		outResponder = new TR_Responder_ATK;
	}
	
	if(outResponder != NULL){
		err = outResponder->Initialize(inName,inInitialNumRequests,inMinFreeRequests,
								inGrowSize,inMaxNumberRequests,
								inMaxRequestBufferSize,inRequestAllocator);
	}
	
	return err;
}



TR_RequestCookie	TR_RequestIterator::GetNext()
{
	TR_RequestCookie			result = kInvalidRequestCookie;
	TR_IncommingRequest_Common*	next = fIter.GetNext();
	if(next != NULL){
		result = next->GetCookie();
	}
	
	return result;
}


void				TR_RequestIterator::Reset()
{
	fIter.Reset();
}


TR_RequestIterator::TR_RequestIterator(TR_Responder& inResponder)
	:	fIter(((TR_Responder_Common*)&inResponder)->fIncommingQueue)
{
	
}


bool				TR_RequestIterator::MoreQ()
{
	return fIter.MoreQ();
}




void			TR_ResponderEventReceiver::AttachTo(TR_Responder* inResponder)
{
	if(fResponder != NULL){
		fResponder->fClients.Dequeue(this);
		fResponder = NULL;
	}
	
	fResponder = (TR_Responder_Common*)inResponder;
	if(fResponder != NULL){
		fResponder->fClients.Enqueue(this);
	}
}


TR_ResponderEventReceiver::TR_ResponderEventReceiver()
	:	fResponder(NULL)
{
	
}


TR_ResponderEventReceiver::~TR_ResponderEventReceiver()
{
	AttachTo(NULL);
}




TR_Responder::TR_Responder()
{
	
}


TR_Responder::~TR_Responder()
{
	
}


TR_IncommingRequest::TR_IncommingRequest()
{
	
}


TR_IncommingRequest::~TR_IncommingRequest()
{
	
}
