#include "TR_Responder_ATK.h"
#include "TR_Allocator.h"
#include <new.h>
#include "PO_Assert.h"
#include "TR_Buffer.h"
#include "PO_Globals.h"

////////////////////////////////////////////////////////////////
//	PRIVATE INTERFACES
////////////////////////////////////////////////////////////////

static OSErr	NBP_Unregister(NamesTableEntry* nte);
static OSErr 	NBP_Register(StringPtr name, StringPtr type, short socket, Boolean autoUnique, NamesTableEntry* nte, 
							Str32 outName);




////////////////////////////////////////////////////////////////
//	PRIVATE CLASSES
////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////
//	_ATQHandler
//
//	This class was "imported" from ATQHandler (from Hubauer Libraries)
//	to avoid dependancies to that library
//	
//	This class encapsalates interaction with the "AppleTalk Transaction Queue"
//	as described in inside mac networking.
////////////////////////////////////////////////////////////////

class _ATQHandler
{
public:
	_ATQHandler();
	virtual ~_ATQHandler();
	
	
	struct CATQEntry
	{
		ATQEntry		atq;
		long			globals;
		_ATQHandler*	thisObj;
	};
	OSErr	HandleATQEvent(long event,void* param);
protected:
	CATQEntry		fEntry;
	
	struct TNetworkTransition
	{
		Ptr			priv;
		ProcPtr		netValidProc;
		Boolean		newConnectivityQ;
	};
	
	struct TNewCRTrans
	{
		short		newCableLo;
		short		newCableHi;
	};

		// override here
	virtual	OSErr	ATQOpen(MPPPBPtr	pb);							// system task time
	virtual OSErr	ATQPrepareToClose();								// system task time
	virtual OSErr	ATQPermissionToClose(StringPtr& outDenyAppName);	// system task time
	virtual OSErr	ATQCancelClose();									// system task time
	virtual OSErr	ATQNetworkConnectionChange(TNetworkTransition& inParam);	// interrupt!
	virtual OSErr	ATQFlagshipNameChange(ConstStr32Param newName);				// system task time
	virtual OSErr	ATQPermissionToChangeFlagshipName(ConstStr32Param newName,StringPtr& outDenyAppName);
	virtual OSErr	ATQCancelFlagshipNameChange();
	virtual OSErr	ATQCableRangeChange(TNewCRTrans& inParam);		// system task time
	virtual OSErr	ATQCPUSpeedChange();			
};

static OSErr	_PATQHandler(long event,ATQEntryPtr entry,void* param)
{
	OSErr			err = noErr;
	long			saveGlobals = PO_GetGlobals();
	_ATQHandler::CATQEntry*		atEntry = (_ATQHandler::CATQEntry*)entry;
	
	PO_SetGlobals(atEntry->globals);
	err = atEntry->thisObj->HandleATQEvent(event,param);
	PO_SetGlobals(saveGlobals);
	return err;

}

_ATQHandler::_ATQHandler()
{
	fEntry.atq.CallAddr = NewATalkTransitionEventProc(_PATQHandler);
	fEntry.globals = PO_GetGlobals();
	fEntry.thisObj = this;
	LAPAddATQ(&fEntry.atq);
}


_ATQHandler::~_ATQHandler()
{
	LAPRmvATQ(&fEntry.atq);
	DisposeRoutineDescriptor(fEntry.atq.CallAddr);
}


OSErr	_ATQHandler::ATQCableRangeChange(TNewCRTrans& inParam)
{
	(void)inParam;
	
	return noErr;
}


OSErr	_ATQHandler::ATQCPUSpeedChange()
{
	return noErr;
}


OSErr	_ATQHandler::HandleATQEvent(long event,void* param)
{
	OSErr		err = noErr;
	StringPtr	p;
	
	switch(event)
	{
		case	0:		// open
			err = ATQOpen((MPPPBPtr)param);
			break;
			
		case	2:		// prep close
			err = ATQPrepareToClose();
			break;
			
		case	3:		// close perm
			p = (StringPtr)param;
			err = ATQPermissionToClose(p);
			break;
			
		case	4:		// cancel close
			err = ATQCancelClose();
			break;
			
		case	5:		// network transision
			err = ATQNetworkConnectionChange(*((TNetworkTransition*)param));
			break;
			
		case	6:		// flagship change
			err = ATQFlagshipNameChange((StringPtr)param);
			break;
			
		case	7:		// flagship perm
			err = ATQPermissionToChangeFlagshipName((StringPtr)param, (StringPtr)param);
			break;
			
		case	8:		// flagship cancel
			err = ATQCancelFlagshipNameChange();
			break;
			
		case	'rnge':	// cable change
			err = ATQCableRangeChange(*((TNewCRTrans*)param));
			break;
			
		case	'sped':	// cpu speed change
			err = ATQCPUSpeedChange();
			break;
	}
	
	
	return err;
}


OSErr	_ATQHandler::ATQPrepareToClose()
{
	return noErr;
}


OSErr	_ATQHandler::ATQPermissionToClose(StringPtr& outDenyAppName)
{
	(void) outDenyAppName;
	
	return noErr;
}


OSErr	_ATQHandler::ATQPermissionToChangeFlagshipName(ConstStr32Param newName,StringPtr& outDenyAppName)
{
	(void)	newName;
	(void) outDenyAppName;
	
	return noErr;
}


OSErr	_ATQHandler::ATQOpen(MPPPBPtr	pb)
{
	(void)pb;
	return noErr;
}


OSErr	_ATQHandler::ATQNetworkConnectionChange(TNetworkTransition& inParam)
{
	(void)inParam;
	return noErr;
}


OSErr	_ATQHandler::ATQFlagshipNameChange(ConstStr32Param newName)
{
	(void) newName;
	return noErr;
}


OSErr	_ATQHandler::ATQCancelFlagshipNameChange()
{
	return noErr;
}


OSErr	_ATQHandler::ATQCancelClose()
{
	return noErr;

}



////////////////////////////////////////////////////////////////
//	TR_ATQHelper
//
//	This class is a subclass of the generic _ATQHandler
//	that commuicates AppleTalk State changes to the responder class
//
////////////////////////////////////////////////////////////////


class	TR_ATQHelper : public _ATQHandler
{
	public:
		TR_ATQHelper(TR_Responder_ATK*	inResponder);
		virtual ~TR_ATQHelper();
	
	
		virtual	OSErr	ATQOpen(MPPPBPtr	pb);
		virtual OSErr	ATQPrepareToClose();
	
	private:
		TR_Responder_ATK*		fResponder;
		bool					fOpenQ;
};



TR_ATQHelper::TR_ATQHelper(TR_Responder_ATK*	inResponder)
{
	fResponder = inResponder;
	fOpenQ = true;
}

TR_ATQHelper::~TR_ATQHelper()
{

}

OSErr	TR_ATQHelper::ATQOpen(MPPPBPtr	pb)
{
	(void) pb;
	if(!fOpenQ){
		fResponder->ATOpen();
	}
	fOpenQ = true;
	return noErr;
}


OSErr	TR_ATQHelper::ATQPrepareToClose()
{
	if(fOpenQ){
		fResponder->ATClose();
	}
	fOpenQ = false;
	return noErr;
}







////////////////////////////////////////////////////////////////
//	TR_Responder_ATK
//
//	AppleTalk specific implementation of the responder
//
////////////////////////////////////////////////////////////////





TR_Responder_ATK::TR_Responder_ATK()
{
	fATPSocketOpenQ = false;
	fNBPRegisteredQ = false;
	fATQHelper = NULL;
}


TR_Responder_ATK::~TR_Responder_ATK()//Override
{
	delete fATQHelper; fATQHelper = NULL;
	
	CloseBaseResponder();
	
}


void	TR_Responder_ATK::GetNetworkAddress(TR_NetworkAddress& outMyAddress)//Override
{
	UInt16		myNode,myNet;
	GetNodeAddress((short*)&myNode, (short*)&myNet);
	myNode &= 0x00FF;
	
	outMyAddress.InitializeFromATK(myNet,myNode,fATPSocket);
}


OSStatus	TR_Responder_ATK::Initialize
(
	const TR_NetworkName&			 	inName,
	UInt32 								inInitialNumRequests,
	UInt32								inMinFreeRequests,
	UInt32								inGrowSize,
	UInt32								inMaxNumRequests,
	UInt32								inMaxRequestBufferSize,
	TR_Allocator*						inRequestAllocator 
)

{
	OSStatus	err = TR_Responder_Common::Initialize(inName,inInitialNumRequests,
												inMinFreeRequests,inGrowSize,
												inMaxNumRequests,
												inMaxRequestBufferSize,
												inRequestAllocator);
	if(err == noErr){
		fATQHelper = new TR_ATQHelper(this);
	}
	
	return err;
}


TR_IncommingRequest_Common*	TR_Responder_ATK::MakeIncommingRequest(
												TR_Allocator* inAllocator,
												PO_Ref<TR_Buffer>	inReqBuffer)//Override

{
	TR_IncommingRequest_Common*		newRequest = NULL;	
	void*		store = inAllocator->Allocate(sizeof(TR_IncommingRequest_ATK));

	if(store != NULL){
		newRequest = new(store) TR_IncommingRequest_ATK(inReqBuffer, this);
			
	}
	
	return newRequest;
					
}


OSStatus		TR_Responder_ATK::InitNetwork()//Override

{
	OSStatus			err = noErr;
	ATPParamBlock		pb;
	
	pb.ATPatpSocket = 0;
	pb.ATPaddrBlock.aNet = 
	pb.ATPaddrBlock.aNode = 
	pb.ATPaddrBlock.aSocket = 0;
	
	err = POpenATPSkt(&pb,false);
	if(err == noErr){
		fATPSocketOpenQ = true;
		fATPSocket = pb.ATPatpSocket;
		
		Str32		name,type,zone;
		fName.GetATKEntity(name,type,zone);
		err = NBP_Register(name, type, fATPSocket, true, &fNTE, NULL);
		// BUG, if unique name is created, it is not propagated out
		if(err == noErr){
			fNBPRegisteredQ = true;
		}
	}else{
		switch(err){
			case	noDataArea:
			case	tooManySkts:
				err = trErrorTooManyResponders;
				break;
			default:
				err = trErrorNetworkError;
				break;
		}
	}
	
	return err;
}


void			TR_Responder_ATK::CloseNetwork()//Override
{
	if(fNBPRegisteredQ){
		NBP_Unregister(&fNTE);
		fNBPRegisteredQ = false;
	}
	
	if(fATPSocketOpenQ){
		ATPParamBlock		pb;
		
		pb.ATPatpSocket = fATPSocket;
		
		PCloseATPSkt(&pb,false);
		fATPSocketOpenQ = false;
	}
}


void			TR_Responder_ATK::ATClose()//Override

{
	CloseRequests();
	CloseNetwork();
}


void			TR_Responder_ATK::ATOpen()//Override

{
	InitNetwork();
	OpenRequests();
}





////////////////////////////////////////////////////////////////
//	TR_IncommingRequest_ATK
//
//	AppleTalk specific implementation of the IncommingRequest object
//
////////////////////////////////////////////////////////////////




void	TR_IncommingRequest_ATK::SendResponse(PO_Ref<TR_Buffer> inResponseBuffer,void* inUserData,
									bool inAsyncQ)//Override

{
	TR_Responder_ATK*		myResponder = (TR_Responder_ATK*)fResponder;
	
	assert(fState == stateIdle);
	
	fResponseBuffer = inResponseBuffer;
	fResponseUserData = inUserData;
	
	fPB.ATPioCompletion = inAsyncQ?fCompletionUPP:NULL;
	fPB.ATPatpSocket = myResponder->fATPSocket;
	fPB.ATPatpFlags = atpEOMvalue | atpSendChkvalue;
	// keep addrblock from listen
	fPB.ATPbdsPointer = (Ptr)fBDS;
	fPB.ATPnumOfBuffs = BuildBDS((Ptr)fResponseBuffer->GetBuffer(),(Ptr)fBDS,fResponseBuffer->GetLogicalSize());
	fPB.ATPbdsSize = fPB.ATPnumOfBuffs;
	// keep transID from listen
	
	fState = stateResponding;
	PSendResponse(&fPB,true);
	
	if(!inAsyncQ){
		while(fPB.ATPioResult > noErr)
			;
		ResponseComplete(false);
	}
}


OSStatus		TR_IncommingRequest_ATK::Initialize()//Override

{
	OSStatus	err =  TR_IncommingRequest_Common::Initialize();
	if(err == noErr){
		fCompletionUPP = NewATPCompletionProc(ATPCompletionStub);
	}
	
	return err;
}


void			TR_IncommingRequest_ATK::KillResponse()//Override

{
	if(fState == stateResponding){
		ATPParamBlock		pb;
		
		pb.ATPatpSocket = fPB.ATPatpSocket;
		pb.ATPaddrBlock = fPB.ATPaddrBlock;
		pb.ATPtransID = fPB.ATPtransID;
		
		PRelRspCB(&pb,false);	
	}
}


void					TR_IncommingRequest_ATK::ATPCompletion()//Override

{
	if(fState == stateListening){
		fState = stateIdle;
		
		if(fPB.ATPioResult == noErr){
			fReqBuffer->SetLogicalSize(fPB.ATPreqLength);
			TellResponderIGotRequest();
		}
	}else if(fState == stateResponding){
		ResponseComplete(true);	
	}
}


void			TR_IncommingRequest_ATK::StopListening()//Override

{
	assert(fState != stateResponding);
	ATPParamBlock		pb;
	
	pb.ATPaKillQEl = (Ptr)&fPB;
	PKillGetReq(&pb,false);
	// NOTE:  This code asserts that the completion proc will get called with the reqAbortErr
}


OSStatus		TR_IncommingRequest_ATK::Listen()//Override

{
	OSErr		err = noErr;
		
	assert(fState == stateIdle);
	
	if(fState == stateIdle){
		fPB.globals = PO_GetGlobals();
		fPB.thisObj = this;
		
		fPB.ATPioCompletion = fCompletionUPP;
		fPB.ATPatpSocket = ((TR_Responder_ATK*)fResponder)->fATPSocket;
		fPB.ATPreqLength = fReqBuffer->GetPhysicalSize();
		fPB.ATPreqPointer = (Ptr)fReqBuffer->GetBuffer();
		
		fState = stateListening;
		err = PGetRequest(&fPB,true);
		if(err != noErr){
			err = trErrorNetworkError;	
			fState = stateIdle;
		}
	}
	
	return err;
}


TR_IncommingRequest_ATK::TR_IncommingRequest_ATK(PO_Ref<TR_Buffer> inBuffer,
													TR_Responder_Common* inResponder)
	:	TR_IncommingRequest_Common(inBuffer,inResponder)
{
	fCompletionUPP = NULL;	
}





#if	GENERATINGCFM
pascal void		TR_IncommingRequest_ATK::ATPCompletionStub(ATPParamBlock* inPB)
{
	ATPExtPB*		pb = (ATPExtPB*) inPB;

	pb->thisObj->ATPCompletion();
	
}



#else
pascal void		TR_IncommingRequest_ATK::ATPCompletionStub()
{
	ATPExtPB*			pb = (ATPExtPB*)GetA0();
	long				saveGlobals = PO_GetGlobals();
	
	PO_SetGlobals(pb->globals);
	
	pb->thisObj->ATPCompletion();
	
	PO_SetGlobals(saveGlobals);
}
#endif	



TR_IncommingRequest_ATK::~TR_IncommingRequest_ATK()//Override
{
	DisposeRoutineDescriptor(fCompletionUPP);
}

void	TR_IncommingRequest_ATK::GetSender(TR_NetworkAddress& outSender) const//Override

{
	outSender.InitializeFromATK(fPB.ATPaddrBlock.aNet,fPB.ATPaddrBlock.aNode,fPB.ATPaddrBlock.aSocket);
}




////////////////////////////////////////////////////////////////
//	PRIVATE IMPLEMENTATION
//	Pulled from Hubauer Libs to avoid dependancies
////////////////////////////////////////////////////////////////


static OSErr NBP_Register(StringPtr name, StringPtr type, short socket, Boolean autoUnique, NamesTableEntry* nte, Str32 outName)
{
	(void)autoUnique;
	
	OSErr			err = noErr;
	MPPParamBlock	pb;
	Str32			stNewName;
	StringPtr		newNamePtr;
	long			count = 1;
	
	if (outName == NULL)
		newNamePtr = stNewName;
	else
		newNamePtr = outName;
	
	PLstrcpy(newNamePtr, name);
	do
	{
		NBPSetNTE((Ptr)nte, newNamePtr, type, "\p*", socket);
		pb.NBPinterval = 5;
		pb.NBPcount = 3;
		pb.NBPentityPtr = (Ptr)nte;
		pb.NBPverifyFlag = true;
		err = PRegisterName(&pb,false);
		if (err == nbpDuplicate)
		{
			Str32		numStr;
			
			PLstrcpy(newNamePtr, name);
			count++;
			NumToString(count, numStr);
			if (newNamePtr[0] + numStr[0] > 31)
				newNamePtr[0] = 31 - numStr[0];
			
			PLstrcat(newNamePtr, numStr);
		}
	}
	while(err == nbpDuplicate);
	
	return err;
}

static OSErr	NBP_Unregister(NamesTableEntry* nte)
{
	MPPParamBlock		pb;
	
	pb.NBP.ioCompletion = NULL;
	pb.NBPentityPtr = ((Ptr)nte)+9;
	
	return PRemoveName(&pb,false);
}


