#include "TR_Responder_Common.h"
#include "TR_Allocator.h"
#include "TR_Buffer.h"
#include "PO_Assert.h"

void		TR_Responder_Common::AttachEventReceiver(TR_ResponderEventReceiver* inEventReceiver)
{
	inEventReceiver->AttachTo(this);
}


TR_IncommingRequest*	TR_Responder_Common::CommitRequest(TR_RequestCookie inCookie)
{
	TR_IncommingRequest*				result = NULL;
	
	
	if(!fKillingRequestsQ){
		TR_IncommingRequest*	thisOne = LocateRequestByCookie(inCookie);
		if(thisOne != NULL){
			if(fIncommingQueue.Dequeue(thisOne) == noErr){
				result = thisOne;

				fActiveQueue.Enqueue(thisOne);
			}
		}
	}
	
	return result;
}








TR_Responder_Common::~TR_Responder_Common()
{
}


void			TR_Responder_Common::InitState()
{
	fKillingRequestsQ = false;
	fCurSize = 0;
	fNeedToGrowQ = false;
}


OSStatus		TR_Responder_Common::OpenRequests()
{
	OSStatus		err = noErr;
	
	PO_TQIterator<TR_IncommingRequest_Common>		iter(fRequestQueue);
	while(iter.MoreQ()){
		TR_IncommingRequest_Common*				thisOne = iter.GetNext();
		
		err = thisOne->Listen();
		if(err != noErr){
			break;
		}
	}
	
	fKillingRequestsQ = false;
	
	return err;
}

void			TR_Responder_Common::GotRequest(TR_IncommingRequest_Common* thisRequest)
{
	static	TR_RequestCookie		nextCookie = 0;
	
	if(!fKillingRequestsQ){
		TR_RequestCookie				thisCookie = ++nextCookie;
		
		fRequestQueue.Dequeue(thisRequest);
		
		if(fRequestQueue.GetNumElems() < fMinFreeRequests){
			fNeedToGrowQ = true;
		}
		
		thisRequest->SetCookie(thisCookie);
		fIncommingQueue.Enqueue(thisRequest);
		
		PO_TQIterator<TR_ResponderEventReceiver>		iter(fClients);
		while(iter.MoreQ()){
			TR_ResponderEventReceiver*		client = iter.GetNext();
			client->NotifyNewRequest(this,thisCookie);
		}
	}
}


void			TR_Responder_Common::ResponseComplete(TR_IncommingRequest_Common* thisRequest,
														bool inNotifyClient, void* inUserData)
{
	if(inNotifyClient){
		PO_TQIterator<TR_ResponderEventReceiver>		iter(fClients);
		while(iter.MoreQ()){
			TR_ResponderEventReceiver*		thisOne = iter.GetNext();
			thisOne->NotifyResponseComplete(this,thisRequest,inUserData);
		}
	}
	
	fActiveQueue.Dequeue(thisRequest);
	fRequestQueue.Enqueue(thisRequest);
	
	if(!fKillingRequestsQ){
		thisRequest->Listen();
	}
}

TR_IncommingRequest_Common*	TR_Responder_Common::LocateRequestByCookie(TR_RequestCookie  inCookie)
{
	TR_IncommingRequest_Common*				result = NULL;
	
	if(inCookie != kInvalidRequestCookie){
		PO_TQIterator<TR_IncommingRequest_Common>		iter(fIncommingQueue);
		while(iter.MoreQ()){
			TR_IncommingRequest_Common*			thisOne = iter.GetNext();
			if(thisOne->GetCookie() == inCookie){
				result = thisOne;
				break;
			}
		}
	}
	
	return result;
}

void			TR_Responder_Common::CloseClients()
{
	PO_TQIterator<TR_ResponderEventReceiver>		iter(fClients);
	while(iter.MoreQ()){
		TR_ResponderEventReceiver*		thisOne = iter.GetNext();
		
		thisOne->AttachTo(NULL);	
	}
}

void		TR_Responder_Common::CloseBaseResponder()
{
	CloseRequests();
	DeleteRequests();
	CloseNetwork();
	CloseClients();
}

OSStatus	TR_Responder_Common::Initialize
(
	const TR_NetworkName&			 	inName,
	UInt32 								inInitialNumRequests,
	UInt32								inMinFreeRequests,
	UInt32								inGrowSize,
	UInt32								inMaxNumRequests,
	UInt32								inMaxRequestBufferSize,
	TR_Allocator*						inRequestAllocator 
)
{
	OSStatus	outErr = noErr;
	
	fName = inName;

	fMinFreeRequests = inMinFreeRequests;
	fGrowSize = inGrowSize;
	fMaxNumRequests = inMaxNumRequests;
	fRequestBufferSize = inMaxRequestBufferSize;
	fNeedToGrowQ = false;
	
	if(inRequestAllocator == NULL){
		inRequestAllocator = gDefaultAllocator;
	}
	fAllocator = inRequestAllocator;
	
	outErr = InitNetwork();
	if(outErr == noErr){
		outErr = CreateRequests(inInitialNumRequests,inMaxRequestBufferSize,false);		
		if(outErr == noErr){
			outErr = OpenRequests();
		}
	}
	
	return outErr;
}







OSStatus		TR_Responder_Common::CreateRequests(UInt32 inNumRequests,
													UInt32 inRequestBufferSize,bool inListenQ)
{
	OSStatus		err = noErr;
	
	for(UInt32 z = 0;z<inNumRequests;z++){
		TR_IncommingRequest_Common*		newRequest;
		PO_Ref<TR_Buffer>				buffer = TR_Buffer::New(inRequestBufferSize, fAllocator);
		
		if(!buffer.IsNullQ()){
			newRequest = MakeIncommingRequest(fAllocator,buffer);
			if(newRequest != NULL){
				err = newRequest->Initialize();
				if(err == noErr){
					fRequestQueue.Enqueue(newRequest);
					newRequest->SetCookie(0);
					if(inListenQ){
						newRequest->Listen();
					}
					
					fCurSize++;
				}
			}else{
				err = trErrorMemFull;
			}
		}else{
			err = trErrorMemFull;
		}
		
		if(err != noErr){
			break;
		}	
	}
	
	
	return err;
}


void			TR_Responder_Common::DeleteRequests()
{
	
	// assert that all requests are in the fRequestQueue and none are listening
	// now kill them off
	
	PO_TQIterator<TR_IncommingRequest>		iter(fRequestQueue);
	while(iter.MoreQ()){
		TR_IncommingRequest*		thisOne = iter.GetNext();
		
		fIncommingQueue.Dequeue(thisOne);
		thisOne->~TR_IncommingRequest();
		fAllocator->Free(thisOne);
	}
}







void			TR_Responder_Common::CloseRequests()
{
	fKillingRequestsQ = true;
	
	// I need to kill all active listeners
	PO_TQIterator<TR_IncommingRequest_Common>		iter(fRequestQueue);
	while(iter.MoreQ()){
		TR_IncommingRequest_Common*				thisOne = iter.GetNext();
		
		thisOne->StopListening();
	}
	
	// I need to kill all pending incomming requests
	PO_TQIterator<TR_IncommingRequest_Common>		iter2(fIncommingQueue);
	while(iter2.MoreQ()){
		TR_IncommingRequest_Common*				thisOne = iter2.GetNext();
		
		if(fIncommingQueue.Dequeue(thisOne) == noErr){
			thisOne->CancelRequest();
		}
	}
	
	// wait for active requests to finish
	while(fActiveQueue.GetNumElems() > 0){
		PO_TQIterator<TR_IncommingRequest_Common>	iter3(fActiveQueue);
		while(iter3.MoreQ()){
			TR_IncommingRequest_Common*	thisOne = iter3.GetNext();
			thisOne->KillResponse();		
		}	
	}

}



TR_Responder_Common::TR_Responder_Common
(
)
{
	InitState();
		
}


void		TR_Responder_Common::RemoveEventReceiver(TR_ResponderEventReceiver* inEventReceiver)
{
	inEventReceiver->AttachTo(NULL);
}


bool		TR_Responder_Common::QueryRequestSender(TR_RequestCookie inCookie,TR_NetworkAddress& outSender)
{
	TR_IncommingRequest*		result = LocateRequestByCookie(inCookie);
	if(result != NULL){
		result->GetSender(outSender);
		return true;
	}else{
		return false;
	}
}


bool		TR_Responder_Common::QueryRequestData(TR_RequestCookie inCookie,PO_Ref<TR_Buffer>& outRequestData)
{
	TR_IncommingRequest*		result = LocateRequestByCookie(inCookie);
	if(result != NULL){
		outRequestData = result->GetRequestBuffer();
		return true;
	}else{
		return false;
	}
}

void	TR_Responder_Common::Idle()//Override
{
	if(fNeedToGrowQ){
		UInt32			sizeToGrow = fGrowSize;
		
		if(fCurSize + sizeToGrow > fMaxNumRequests){
			sizeToGrow = fMaxNumRequests - fCurSize;
		}
		
		if(sizeToGrow > 0){
			OSStatus		err = CreateRequests(sizeToGrow,fRequestBufferSize,true);
			// BUG!  what to do with an error here.
			//		 probably should have a "can't grow" proc like the PO_ObjectPool class
		}
		
		fNeedToGrowQ = false;
	}
}


UInt32	TR_Responder_Common::GetNumFree()//Override
{
	return fRequestQueue.GetNumElems();
}


UInt32	TR_Responder_Common::GetPoolSize()//Override

{
	return fCurSize;
}










void			TR_IncommingRequest_Common::CancelRequest()
{
	assert(fState == stateIdle);
	ResponseComplete(false);
}





TR_IncommingRequest_Common::TR_IncommingRequest_Common(PO_Ref<TR_Buffer> inBuffer,
														TR_Responder_Common* inResponder)
{
	fReqBuffer = inBuffer;
	fResponder = inResponder;
	fState = stateIdle;
}


TR_IncommingRequest_Common::~TR_IncommingRequest_Common()
{
	assert(fState == stateIdle);
}









void			TR_IncommingRequest_Common::ResponseComplete(bool inNotifyClientQ)
{
	fState = stateIdle;
	fResponseBuffer = NULL;		// release it
	fResponder->ResponseComplete(this,inNotifyClientQ,fResponseUserData);
}

void				TR_IncommingRequest_Common::TellResponderIGotRequest()
{
	fResponder->GotRequest(this);
}



OSStatus		TR_IncommingRequest_Common::Initialize()
{
	return noErr;
}


PO_Ref<TR_Buffer>	TR_IncommingRequest_Common::GetRequestBuffer() const
{
	return fReqBuffer;
}

