#pragma once
#ifndef INCLUDED_TR_RESPONDER_COMMON
#define INCLUDED_TR_RESPONDER_COMMON
////////////////////////////////////////////////////////////////////////
//	
//	TR_Responder_Common
//
//	This class defines the common implementation
//	for all responder types
//
//	THIS IS NOT A PUBLIC INTERFACE FILE.
//	
//////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////
//	INCLUDES
//////////////////////////////////////////////////////////////////////////

#include "TR_Responder.h"


//////////////////////////////////////////////////////////////////////////
//	REFERENCES BY NAME
//////////////////////////////////////////////////////////////////////////

class TR_IncommingRequest;


//////////////////////////////////////////////////////////////////////////
//	PRIVATE CLASSES
//////////////////////////////////////////////////////////////////////////

class TR_Responder_Common : public TR_Responder
{
	public:
	
		
		TR_Responder_Common();
		virtual ~TR_Responder_Common();
		
		
		virtual OSStatus	Initialize
		(
			const TR_NetworkName&			 	inName,
			UInt32 								inInitialNumRequests,
			UInt32								inMinFreeRequests,
			UInt32								inGrowSize,
			UInt32								inMaxNumRequests,
			UInt32								inMaxRequestBufferSize,
			TR_Allocator*						inRequestAllocator 
		);
		
		
		// request notification
		
		virtual void		AttachEventReceiver(TR_ResponderEventReceiver* inEventReceiver);
		virtual void		RemoveEventReceiver(TR_ResponderEventReceiver* inEventReceiver);

		// query a request new request
		

		virtual bool		QueryRequestData(TR_RequestCookie inCookie,PO_Ref<TR_Buffer>& outRequestData);
		virtual bool		QueryRequestSender(TR_RequestCookie inCookie,TR_NetworkAddress& outSender);
		
		// commit to handling a new request
		
		virtual TR_IncommingRequest*	CommitRequest(TR_RequestCookie inCookie);

		virtual void	Idle();
		virtual UInt32	GetPoolSize();
		virtual UInt32	GetNumFree();

	protected:
		void		CloseBaseResponder();

		
		OSStatus	OpenRequests();	
		void		CloseRequests();

		virtual OSStatus		InitNetwork() = 0;
		virtual void			CloseNetwork() = 0;

		virtual TR_IncommingRequest_Common*	MakeIncommingRequest(
												TR_Allocator* inAllocator,
												PO_Ref<TR_Buffer>	inReqBuffer) = 0;

		TR_NetworkName	fName;
	private:
		friend class	TR_RequestIterator;
		friend class 	TR_IncommingRequest_Common;
		friend class	TR_ResponderEventReceiver;
		
		OSStatus		CreateRequests(UInt32 inNumRequests,
													UInt32 inRequestBufferSize,bool inListenQ);
		void			DeleteRequests();
		
		void			InitState();
			
		void			GotRequest(TR_IncommingRequest_Common* thisRequest);
		void			ResponseComplete(TR_IncommingRequest_Common* thisRequest,
														bool inNotifyClient, void* inUserData);
		TR_IncommingRequest_Common*	LocateRequestByCookie(TR_RequestCookie  inCookie);
		
		void			CloseClients();
		
		PO_TQueue<TR_IncommingRequest>					fRequestQueue;
		PO_TQueue<TR_IncommingRequest>					fIncommingQueue;
		PO_TQueue<TR_IncommingRequest>					fActiveQueue;
		PO_TQueue<TR_ResponderEventReceiver>			fClients;
		bool											fKillingRequestsQ;
		TR_Allocator*									fAllocator;
		
		UInt32											fMinFreeRequests;
		UInt32											fGrowSize;
		UInt32											fMaxNumRequests;
		UInt32											fRequestBufferSize;
		bool											fNeedToGrowQ;
		UInt32											fCurSize;

		



};


class TR_IncommingRequest_Common : public TR_IncommingRequest
{
	friend class TR_Responder_Common;
	friend class TR_RequestIterator;
	public:
	
		virtual PO_Ref<TR_Buffer>	GetRequestBuffer() const;
		
		virtual void				CancelRequest();
 
	
	protected:
	
		enum{
			stateListening,
			stateIdle,
			stateResponding
		};
			
		UInt32			GetState()				{ return fState;}
		
		TR_IncommingRequest_Common(PO_Ref<TR_Buffer> inBuffer,
														TR_Responder_Common* inResponder);
		virtual ~TR_IncommingRequest_Common();
		
		void				TellResponderIGotRequest();
		
		virtual OSStatus	Initialize();
		virtual OSStatus	Listen() = 0;
		virtual void		StopListening() = 0;
		virtual void		KillResponse() = 0;
		
		void				ResponseComplete(bool inNotifyClientQ);
		void				SetCookie(TR_RequestCookie inCookie) { fCookie = inCookie;}
		TR_RequestCookie	GetCookie()							 { return fCookie; }

		
		PO_Ref<TR_Buffer>		fReqBuffer;
		TR_Responder_Common*	fResponder;
		TR_RequestCookie		fCookie;
		UInt32					fState;
		PO_Ref<TR_Buffer>		fResponseBuffer;
		void*					fResponseUserData;


};



#endif
