#pragma once
#ifndef INCLUDED_TR_DEFERREDACTIONS
#define INCLUDED_TR_DEFERREDACTIONS
////////////////////////////////////////////////////////////////////////
//	
//	TR_DeferredActions
//
//	This class manages a "pool" of "action elements" that used to 
//	"defer" actions to a safe "idle" time.
//	
//	This is useful if you have to respond to an "interupt" but the
//	action you need to perform has to be deferred until a "memory safe"
//	time.
//	
//////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////
//	INCLUDES
//////////////////////////////////////////////////////////////////////////

#include "PO_TObjectPool.h"
#include "PO_TQueue.h"


//////////////////////////////////////////////////////////////////////////
//	PUBLIC CLASSES
//////////////////////////////////////////////////////////////////////////

class	TR_DeferredActions
{
	public:
	
		TR_DeferredActions
		(
			UInt32	inInitialElems,			// initial number of elements
			UInt32	inMinNumFree,			// number of free that must be maintained
											// after which the inGrowSize is allocated
			UInt32	inGrowSize,				// grow increment if minNumFree is exceded
			UInt32	inMaxSize				// max number of elements to grow to.
		);
		
		~TR_DeferredActions();
		
		void	Idle();						// must idle at memory safe time
		
		
		typedef pascal void	(*ActionProc)(void* param1,void* param2);
		
		bool	DeferAction(ActionProc inProc,void* param1,void* param2);


	private:
		
		struct ActionElem : public PO_TQueueElem<ActionElem>
		{
			ActionProc	fProc;
			void*		fParam1;
			void*		fParam2;
		};
		
		PO_TObjectPool<ActionElem>		fElemPool;
		PO_TQueue<ActionElem>			fQueue;
};




#endif

