#pragma once
#ifndef INCLUDED_NETWORKTYPES
#define INCLUDED_NETWORKTYPES

////////////////////////////////////////////////////////////////
//
//	MODULE:			TR_NetworkTypes
//	
//	DESCRIPTION:	TR_NetworkTypes contains the type primitives
//					and error codes for the TR (Transaction) library
//
//	AUTHORS:		Bill Hubauer (BH)
//
//	HISTORY:
//		WHEN		WHO		WHAT
//		----		---		----
//		5/22		BH		Created and Debugged
//
////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////
//	INCLUDES
////////////////////////////////////////////////////////////////

#include "PO_Stream.h"


////////////////////////////////////////////////////////////////
//	PUBLIC ERROR CODES
////////////////////////////////////////////////////////////////

enum
{
	trErrorBadName = 2000,			// TR_NetworkAddress::ConfirmName failed
	trErrorNameNotFound,			// TR_NetworkAddress::InitializeFromName failed
	trErrorNetworkError,			// Generic unknown error.  (ie. AppleTalk not loaded)
	trErrorNoResponse,				// TR_OutgoingRequest failed to complete because Responder didn't respond.
	trErrorTooManyRequests,			// Too many active TR_OutgoingRequest objects
	trErrorRequestTooBig,			// Request buffer is too big for network.
	trErrorTooManyResponders,		// Too many active TR_Responder objects
	
	trErrorRequestCanceled = userCanceledErr,	// user canceled TR_OutgoingRequest
	trErrorMemFull = memFullErr
	
};


/////////////////////////////////////////
//	CLASS:  TR_NetworkName
/////////////////////////////////////////

typedef OSType	TR_NetworkNameType;
enum{
	kNameTypeAppleTalk = 'NATK',
	kNameTypeIPName = 'NIP ',
	kNameTypeIPAddress = 'AIP ',
	kNameTypeUndefined = '????'
};

class TR_NetworkName
{ 
	public:
		TR_NetworkName(ConstStr32Param inNBPName,ConstStr32Param inNBPType,
						ConstStr32Param inNBPZone);
		TR_NetworkName(ConstStr32Param inNBPType);	// use sharing setup name and local zone
		
		TR_NetworkName(ConstStr63Param inIPNameOrAddress,UInt16 inIPPort);
		TR_NetworkName(UInt32 inIPAddress,UInt16 inIPPort);
		TR_NetworkName(PO_StreamRef  inConstructFromStream);
		TR_NetworkName();		// undefined
		
		OSStatus			WriteToStream(PO_StreamRef inStream);
	
		TR_NetworkNameType	GetType() const			{return fType;}
		
		bool				GetATKEntity(Str32 outName,Str32 outType,Str32 outZone) const;
		bool				GetIPName(Str255 outName,UInt16& outPort) const;
		bool				GetIPAddress(UInt32& outHost,UInt16& outPort) const;
		
	private:
		// representation
		
		TR_NetworkNameType		fType;
		union {
			EntityName			fATK;
			
			struct {
				UInt16			fPort;
				Str63			fName;
			}fIPName;
			
			struct {
				UInt16			fPort;
				UInt32			fAddress;
			}fIPAddress;
		};
	
};
	

/////////////////////////////////////////
//	CLASS: TR_NetworkAddress
/////////////////////////////////////////
	
typedef OSType	TR_NetworkAddressType;
enum {
	kAddressTypeUndefined = '????',
	kAddressTypeAppleTalk = 'ATK ',
	kAddressTypeIP = 'IP  '
};


class TR_NetworkAddress
{
	public:
		TR_NetworkAddress();
		
		OSStatus				InitializeFromName(const TR_NetworkName& inName);
		void					InitializeFromATK(UInt16 inNet,UInt16 inNode,UInt16 inSocket);
		void					InitializeFromIPAddress(UInt32 inHost,UInt16 inPort);
		
		OSStatus				ConfirmName(const TR_NetworkName& inName);
				
		TR_NetworkAddressType	GetType() const		{return fType;}
		
		bool					GetATKAddress(AddrBlock& outAddress,
												UInt16& outTimeoutSeconds,UInt16& outRetries) const;
		bool					GetIPAddress(UInt32& outHost,UInt16& outPort) const;
	 
	private:
		TR_NetworkAddressType	fType;
		union {
			struct {
				AddrBlock	fAddr;
				UInt16		fTimeout;
				UInt16		fRetries;
			}atk;
			
			struct {
				UInt32		fHostIP;
				UInt16		fHostPort;
			}ip;
		};
};



#endif

