#pragma once
#ifndef INCLUDED_TR_BUFFER
#define INCLUDED_TR_BUFFER
////////////////////////////////////////////////////////////////
//
//	CLASS:			TR_Buffer
//	
//	DESCRIPTION:	TR_Buffer is a ref counted buffer for use
//					with the TR (Transaction) library.
//
//	AUTHORS:		Bill Hubauer (BH)
//
//	HISTORY:
//		WHEN		WHO		WHAT
//		----		---		----
//		5/22		BH		Created and Debugged
//
////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////
//	INCLUDES
////////////////////////////////////////////////////////////////

#include "PO_RefCounted.h"


////////////////////////////////////////////////////////////////
//	REFERENCES BY NAME
////////////////////////////////////////////////////////////////

class	TR_Allocator;



////////////////////////////////////////////////////////////////
//	PUBLIC CLASSES
////////////////////////////////////////////////////////////////

class TR_Buffer : public PO_RefCounted
{
	public:
		static TR_Buffer*	New(UInt32 inBufferSize,TR_Allocator* inAllocator = NULL);
		virtual 			~TR_Buffer();
		
		static UInt32		GetBufferSizeOverhead() { return sizeof(TR_Buffer); }
		
		void				operator delete(void* inObj);
		
		void*				GetBuffer()							{ return fMem; }
		UInt32				GetPhysicalSize()					{ return fSize; }
		UInt32				GetLogicalSize()					{ return fLogSize; }
		
		void				SetLogicalSize(UInt32 inNewSize)	{ fLogSize = inNewSize;}
 
	protected:
		TR_Buffer();
		void*	operator new(UInt32 inObjectSize,TR_Allocator& inAllocator,UInt32 inBufferSize);

		UInt32			fSize;
		UInt32			fLogSize;
		TR_Allocator*	fAllocator;
		void*			fMem;
};



template <class T>
class TR_TBuffer : public TR_Buffer
{
	public:
		
		void*	operator new(UInt32 inObjectSize,TR_Allocator& inAllocator)
		{
			return TR_Buffer::operator new(inObjectSize,inAllocator,sizeof(T));
		}
		
		operator T&()		{ return *((T*)GetBuffer()); }
		operator T*()		{ return (T*)GetBuffer(); }
};



#endif