#pragma once
#ifndef INCLUDED_TR_BUFFERPOOL
#define INCLUDED_TR_BUFFERPOOL
////////////////////////////////////////////////////////////////////////
//	
//	TR_BufferPool
//
//	This class manages a pool of TR_Buffer objects
//	The cool thing about this is that since the TR_Buffer objects
//	are ref counted, as soon as one goes out of scope, its automatically 
//	placed back into the pool.
//	
//////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////
//	INCLUDES
//////////////////////////////////////////////////////////////////////////

#include "TR_PoolAllocator.h"
#include "PO_Ref.h"


//////////////////////////////////////////////////////////////////////////
//	REFERENCED BY NAME
//////////////////////////////////////////////////////////////////////////

class TR_Buffer;



//////////////////////////////////////////////////////////////////////////
//	PUBLIC CLASSES
//////////////////////////////////////////////////////////////////////////

class TR_BufferPool
{
	public:
	
		// fixed size pool
		
		TR_BufferPool(UInt32 inMaxBufferSize);
		
		// growable pool
		
		TR_BufferPool(UInt32 inMaxBufferSize,UInt32 inInitialSize,UInt32 inMinFree,
						UInt32 inGrowSize,UInt32 inMaxSize);
		~TR_BufferPool();
		
		OSStatus			IncreasePoolSize(UInt32		inNumberOfBuffers);
		
		UInt32				GetPoolSize();
		UInt32				GetNumFree();
		UInt32				GetHighestUsage();
		
		bool				Allocate(PO_Ref<TR_Buffer>& outBuffer,UInt32 inReqSize);
		
	private:
		TR_PoolAllocator	fAllocator;
		UInt32				fBufferSize;

		

};


#endif