#pragma once
#ifndef INCLUDED_TR_REQUESTOR
#define INCLUDED_TR_REQUESTOR
////////////////////////////////////////////////////////////////
//
//	CLASS:			TR_OutgoingRequest
//	
//	DESCRIPTION:	TR_OutgoingRequest is the public interface
//					for sending requests to a responder
//
//	AUTHORS:		Bill Hubauer (BH)
//
//	HISTORY:
//		WHEN		WHO		WHAT
//		----		---		----
//		5/22		BH		Created and Debugged
//
////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////
//	INCLUDES
////////////////////////////////////////////////////////////////

#include "PO_Ref.h"


////////////////////////////////////////////////////////////////
//	REFERENCES BY NAME
////////////////////////////////////////////////////////////////

class	TR_NetworkAddress;
class	TR_Buffer;
class	TR_Allocator;

////////////////////////////////////////////////////////////////
//	PUBLIC CLASSES
////////////////////////////////////////////////////////////////

class TR_OutgoingRequest;

class TR_RequestFactory
{
	public:
		
		TR_RequestFactory();
		~TR_RequestFactory();
		

		static UInt32	MaxRequestSizeForAddressType(UInt32 inAddressType);
		static UInt32	sizeof_TR_OutgoingRequest(UInt32 inAddressType);
		
			
		typedef UInt32		Priority;
		enum{
			kLowPriority,
			kNormalPriority,
			kHighPriority
		};
		
		 
		static TR_OutgoingRequest*		New
		(
 			const 	TR_NetworkAddress& 	inAddress,
			PO_Ref<TR_Buffer>			inReqData,
			PO_Ref<TR_Buffer>			inRespBuffer,
			TR_Allocator*				inAllocator = NULL,
			Priority					inPriority = kNormalPriority
		);
};




class TR_OutgoingRequest
{
	public:
		TR_OutgoingRequest();
		virtual ~TR_OutgoingRequest();
		
				
		// async operation
		
		typedef pascal void (*NotifyProc)(void* userData,TR_OutgoingRequest* inRequest);
		virtual void	NotifyWhenComplete(NotifyProc inProc,void* inUserData) = 0;

		// sync operation
		
		virtual void	WaitUntilComplete() = 0;
		
		// polling
		
		virtual Boolean	IsCompleteQ() = 0;
		
		// caneling
		
		virtual void	Cancel() = 0;		// not interrupt safe
		
		// results query
		
		virtual OSStatus			GetResult() = 0;
		virtual PO_Ref<TR_Buffer>	GetResponse() = 0;
	
	
};




#endif