//******************************************************************************
//******************************************************************************
/*
	File:		Brickout.h

	Contains:	Header for a sample dcmd which plays brickout.

	Written by:	Andy Bachorski and Nat McCully

	Copyright:	 1998 by Apple Computer, Inc., All Rights Reserved.

	Change History (most recent first):
	
		<1> 98-6-19		Written for the MacHack 98 Hack Contest.

	The following MPW commands will build the dcmd and copy it to the "Debugger Prefs" file
	in the System folder. The dcmd's name in MacsBug will be the name of the file built by
	the Linker.

	set MacsBugLibs "Tools:OtherSDKs:Building dcmds:MacsBug 6.5.4a3:MacsBug 6.5.4a3:Building dcmds:dcmd Libraries:"
	SC BrickOut.c  -i "Tools:OtherSDKs:Building dcmds:MacsBug 6.5.4a3:MacsBug 6.5.4a3:Building dcmds:dcmd Includes:"
	Link -o BrickOut -sg Main=STDCLIB,STDIO,SANELIB BrickOut.o 																
		"{MacsBugLibs}dcmdGlue.a.o"
	BuildDcmd BrickOut 195 -format3
	Echo 'include "BrickOut";'	|	Rez -a -o "{SystemFolder}Debugger Prefs"

*/


//******************************************************************************

#ifndef _MacBrickOut_
#define _MacBrickOut_

#include <Memory.h>
#include <Quickdraw.h>
#include <Types.h>

//#include <string.h>

#include "dcmd.h"

//******************************************************************************

#define	MaxSpeed		4

#define	BricksH			numBricksX
#define	BricksV			numBricksY

#define	WidthCalc		brickWidth
#define	HeightCalc		brickHeight

#define kHit			1
#define	kSafe			0

#define	KeyMapLoMem 	((unsigned char *)0x174)
#define KeyIsDown(key)	(( KeyMapLoMem[ key >> 3 ] >> ( key & 7)) &1)

#define	aKey			0x00
#define	sKey			0x01
#define qKey			0x0C
#define	fourKey			0x56
#define	sixKey			0x58

#define kSplashColorBlack	0x15
#define kSplashColorBlue	0x02
#define kSplashColorLtBlue	0x03
#define kSplashColorGreen	0x04
#define kSplashColorPurple	0x01
#define kSplashColorRed		0x06
#define kSplashColorYellow	0x05

//******************************************************************************

enum {
	kScreenLines	= 42,			//	Number of lines to scroll to clear MacsBug screen
	numRows			= 0x01a8,
	topBorder		= 8,
	bottomBorder	= 8,
	sideBorder		= 4,
	leftEdge		= 74,
	brickWidth		= 56,				// byte * 8 = 48 pixels
	halfBrickWidth	= brickWidth / 2,
	brickHeight		= 12,				// 1 row = 1 pixel * 12 = 12 pixels
	halfBrickHeight	= brickHeight / 2,
	numBricksX		= 10,
	numBricksY		= 6,
	kWhite			= 0x00,
	kBlack			= 0x01,
	kRed			= 0x02,
	ballSize		= brickHeight,
	halfBallSize	= ballSize / 2,
	paddleWidth		= brickWidth,
	halfPaddleWidth	= paddleWidth / 2,
	paddleHeight	= brickHeight,
	halfPaddleHeight= brickHeight / 2,
	paddleMoveAmt	= 2
};

//******************************************************************************

long			gRunning;
long			gFellOff;

UInt32		gScreenBase;
UInt32		gVideoRowBytes;
UInt32		gScreenBaseOffset;

UInt16		gScreenRowBytes;
UInt32		gDrawBoundsTop;
UInt32		gDrawWidth;

UInt16		gBrickLeft;
UInt16		gBrickRight;

UInt16		gBallStartX;
UInt16		gBallStartY;

Rect		ballRect;
Rect		oldBallRect;

UInt16		gPaddleStartX;
UInt16		gPaddleStartY;

Rect		paddleRect;
Rect		oldPaddleRect;

Point		speed;


char 		blackBall[];
char 		whiteBall[];

short		BrickState[BricksH][BricksV];
Rect		Bricks[BricksH][BricksV];

Point		gSplashDimensions;
Rect		gSplashRect;
char		gSplashBits[];

//******************************************************************************

pascal	void CommandEntry( dcmdBlock* paramPtr );

void	ScrollDisplayOff( void );		//	Scroll the current contents of the Macs

void	DoBrickOut( void );				//	The 'event loop' for the game

void	PauseWithPolling( void );		//	 A way to throttle the speed of the game
void	PauseAndUpdatePaddle( long ticksToWait );
Boolean InitGlobals( void );
long	RowOffset( long y );
void	NumberToHex( UInt32 number, Str255 hex );
void	InitBallStart( void );
void	DrawBorder( void );
void	DrawBricks( void );
void	DrawBrick( short xLoc, short yLoc, Rect *rect );
void	FadeToBlack( void );
void	CheckCollide( void );
char *	XYtoAddress( long xLoc, long yLoc );
void	DrawBall( Rect *ballRectPtr, char * ballBits );
void	UpdateBall( void );
void	DrawPaddle( Rect *paddleRectPtr );
void	UpdatePaddles( void );
void	DrawSplashScreen( Rect *splashRectPtr, char * splashBits, Point * dimensions );


//******************************************************************************
#endif // _MacBrickOut_
