#include "CCDDrive.h"

//#include "CCompactDisc.h"
//#include "Helpers.h"

#include <Devices.h>
#include <LowMem.h>
#include "Str255.h"
#include <UException.h>
const short kReadTOCControlCode = 100;

namespace {

bool
GoodBCD(UInt8 bcd)
{
	return bcd <= 0x99 && (bcd & 0xF) <= 9;
}

UInt8
DecodeBCD(UInt8 bcd)
{
	Assert_(GoodBCD(bcd));
	return static_cast<UInt8>((bcd >> 4) * 10 + (bcd & 0xF));
}

}

CDDrive::CDDrive(short refNum)
	: mRefNum(refNum)
{
}

bool
CDDrive::AudioTrack(unsigned long trackStart)
{
	unsigned controlField = (trackStart >> 24) & 0xF;
	switch (controlField & ~2) {
		case 0: // 2 audio channels without pre-emphasis
		case 1: // 2 audio channels with pre-emphasis
		case 8: // 4 audio channels without pre-emphasis
		case 9: // 4 audio channels with pre-emphasis
			return true;
	}
	return false;
}

void
CDDrive::Eject() const
{
	const int kMaxVolumesPerCD = 8;
	short numVolumes = 0;
	short volumes[kMaxVolumesPerCD];
	
	HParamBlockRec parameters;
	HVolumeParam& volume = parameters.volumeParam;
	volume.ioNamePtr = nil;
	for (volume.ioVolIndex = 1; ; ++volume.ioVolIndex) {
		OSErr error = ::PBHGetVInfoSync(&parameters);
		if (error == nsvErr)
			break;
		if (error == noErr && volume.ioVDRefNum == mRefNum)
			volumes[numVolumes++] = volume.ioVRefNum;
	}
	
	if (numVolumes == 0) // The drive wasn't mounted, so tell the drive to eject.
		::Control(mRefNum, ejectCode, nil);
	else {
		if (::Eject(nil, volumes[0]) == noErr)
			for (int i = 0; i < numVolumes; i++)
				::UnmountVol(nil, volumes[i]);
	}
}

bool
CDDrive::Empty() const
{
	ParamBlockRec parameters;
	CntrlParam& control = parameters.cntrlParam;
	control.ioVRefNum = 1;
	control.ioCRefNum = mRefNum;
	control.csCode = kReadTOCControlCode;
	control.csParam[0] = 1;
	return PBControlSync(&parameters) != noErr;
}

CDDrive::Iterator::Iterator()
	: mCurrentUnitNumber(0)
{
	Skip();
}

void
CDDrive::Iterator::Advance()
{
	mCurrentUnitNumber += 1;
}

void
CDDrive::Iterator::Skip()
{
	// Skip's job is to advance past any unit numbers that are not
	// valid CDs. It doesn't advance at all if the current unit number
	// is a valid CD.
	while (More() && !IsCD())
		Advance();
}

bool
CDDrive::Iterator::More() const
{
	return mCurrentUnitNumber < LMGetUnitTableEntryCount();
}

CDDrive
CDDrive::Iterator::Next()
{
	short current = CurrentRefNum();
	Advance();
	Skip();
	return current;
}

short
CDDrive::Iterator::CurrentRefNum() const
{
	return static_cast<short>(~mCurrentUnitNumber);
}

bool
CDDrive::Iterator::IsCD() const
{
	DCtlHandle handle = GetDCtlEntry(CurrentRefNum());
	if (handle == nil)
		return false;
	DRVRHeaderPtr driver(reinterpret_cast<DRVRHeaderPtr>(handle[0]->dCtlDriver));
	if (handle[0]->dCtlFlags & dRAMBasedMask)
		driver = *reinterpret_cast<DRVRHeaderHandle>(driver);
	return IsCDDriverName(driver->drvrName);
}

bool
CDDrive::Iterator::IsCDDriverName(ConstStr255Param name)
{
	// Name must start with ".AppleCD" or ".XppleCD" to be a CD driver.
	std::string prefix(AsString(name));
	if (prefix.size() < 8)
		return false;
	prefix.erase(8);
	return prefix == ".AppleCD" || prefix == ".XppleCD";
}

unsigned
CDDrive::NumDrives()
{
	unsigned num = 0;
	for (Iterator drives; drives.More(); drives.Next())
		num += 1;
	return num;
}
