#pragma once

#include <vector>

namespace IRTransfer {

typedef struct {
	long	type;
	long	size;
} PacketHeader;

typedef struct {
	long	currX;
	long	currY;
	long	deltaX;
	long	deltaY;
} MouseData;

typedef struct {
	long	buttons;
} Buttons;

typedef struct {
	long	width;
	long	height;
	long	rowBytes;
	long	pixelDepth;
	Uint8	data[4];
} GraphicsData;

const int GraphicsDataHeaderSize = sizeof(GraphicsData) - 4;

}

class CIRTransfer {
public:
	enum {mousePacket=0x13330001, graphicsPacket, requestGraphicsPacket, buttonsPacket};
	CIRTransfer();
	~CIRTransfer();	
	
	bool OpenIR();
	void CloseIR();
	
	void SendPacket(const IRTransfer::MouseData&);
	void SendPacket(const IRTransfer::GraphicsData&);
	void SendPacket(int packetType, const void *packetData, Int32 packetLength);
	bool ReceivePacket(int& packetType, void *packetData, Int32& packetLength);
	bool OutOpen() const {return fInRefNum;}
	bool InOpen() const {return fOutRefNum;}
	int OutStatus() const;
	long LastPacketTick() const {return fLastPacketTick;}

private:

	bool	fOpened;
	bool	fCantOpen;
	short	fInRefNum;
	short	fOutRefNum;
	long	fLastPacketTick;
};