#pragma once

extern void ClearBlock(void *p, int length);

template <class T> inline void Clear(T &foo)
{
	ClearBlock(&foo, sizeof(foo));
}

#include <PP_Prefix.h>
#include <LFile.h>
#include <UGWorld.h>
#include <LString.h>

class LView;
class LListBox;

template <class T>
inline void (Clamp) (T& x, T minValue, T maxValue)
{ 
	if (x < minValue) x = minValue;
	if (x > maxValue) x = maxValue;
}

template <class T>
inline void (ByteClamp) (T& x)
{ 
	if (x < 0) x = 0;
	if (x > 255) x = 255;
}

template <class T>
inline void ForgetObject(T* & x)
{
	T *temp = x;
	x = 0;
	delete temp;
}

template <class T>
void (FindPane) (LView *view, const PaneIDT& paneID, T** result)
{
	*result = 0;
	T* temp = dynamic_cast<T*>(view->FindPaneByID(paneID));
	ThrowIfNil_(temp);
	*result = temp;
}

extern void Normalize(const FSSpec& inFileSpec, FSSpec* outSpec);
extern void GetPreferencesFolder(FSSpec& spec);
extern void GetTemporaryFile(FSSpec *outSpec);
extern void GetTemporaryFileOnVolume(short vRefNum, FSSpec *outSpec);
extern void GetTemporaryFileNear(const FSSpec& inSpec, FSSpec *outSpec);
extern bool GetTempFileNamed(ConstStringPtr fileName, FSSpec *outSpec);
extern Ptr LockGWorldPixels(LGWorld* world);
extern void UnlockGWorldPixels(LGWorld* world);
extern int GWorldRowBytes(LGWorld* world);
extern void WriteString(LStream& inStream, const string& theString);
extern void WriteStringAsText(LStream& inStream, const string& theString);
extern string ReadString(LStream& inStream);
extern int cmp_nocase(const string &s, const string& s2);
extern string ConvertErrorToString(ExceptionCode errorCode);
extern string DescribeFolder(const FSSpec& thisFolder);
extern string DescribeFile(const FSSpec& thisFolder);
extern vector<string> ExpandStringList(const vector<string>& labels, const vector<int>& repeats);
extern void WriteFileSpec(LStream& aStream, const FSSpec& aSpec);
extern bool ReadFileSpec(LStream& aStream, FSSpec& aSpec);

class CFSSpec {
public:
		CFSSpec();
		CFSSpec(const FSSpec& spec);
		
		operator const FSSpec * () const	{ return &fSpec; }
		operator FSSpec *	()		{ return &fSpec; }

private:
	FSSpec fSpec;

};

inline
Boolean operator==(
				const FSSpec&	inLhs,
				const FSSpec&	inRhs)
		{
			return LFile::EqualFileSpec(inLhs, inRhs);
		}

inline
Boolean operator!=(
				const FSSpec&	inLhs,
				const FSSpec&	inRhs)
		{
			return !LFile::EqualFileSpec(inLhs, inRhs);
		}

inline
Boolean operator<(
				const FSSpec&	inLhs,
				const FSSpec&	inRhs)
		{
			LStr255 inLhsStr(inLhs.name);
			return inLhsStr.CompareTo(inRhs.name) < 0;
		}

inline GrafPtr MakeGrafPtr(LGWorld *world)
		{
			return (GrafPtr)world->GetMacGWorld();
		}

inline BitMapPtr MakeBitMapPtr(LGWorld *world)
		{
			GrafPtr ptr = (GrafPtr)world->GetMacGWorld();
			return &ptr->portBits;
		}

inline GrafPtr MakeGrafPtr(StOffscreenGWorld &world)
		{
			return (GrafPtr)world.GetMacGWorld();
		}

extern void FillListBox(LListBox* theBox, const vector<string>& theStrings);

template <class T>
int FindIndex(const vector<T>& theList, const T& theItem)
{
	vector<T>::const_iterator i = find(theList.begin(), theList.end(), theItem);
	ThrowIf_(i == theList.end());
	return i - theList.begin();
}

void ConfirmFreeStackSpace();
bool IsQuietError(ExceptionCode thisError);
