// CopyAction.cp

#ifndef CopyAction_h
#include "CopyAction.h"
#endif
#ifndef CatInfo_h
#include "CatInfo.h"
#endif
#ifndef HaveFinderUpdate_h
#include "HaveFinderUpdate.h"
#endif
#ifndef HaveFinderResolve_h
#include "HaveFinderResolve.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef HaveFinderSelect_h
#include "HaveFinderSelect.h"
#endif
#ifndef FileNotFoundError_h
#include "FileNotFoundError.h"
#endif
#ifndef ConfigureFinderUpdating_h
#include "ConfigureFinderUpdating.h"
#endif

void CopyAction::SetEvent( const AEList& newSelection,
									const AEEvent& copyEvent )
  {
	AEDescriptor selectionCopy( newSelection );
	selectionCopy.Swap( selection );

	AEDescriptor eventCopy( copyEvent );
	eventCopy.Swap( event );
  }

FileLocation CopyAction::FindCopy( const FileLocation& original,
											  Directory at )
  {
	FileLocation newCopy( at, original.name );
	
	if ( !CouldBeCopy( CatInfo( original ), CatInfo( newCopy ) ) )
		throw FileNotFoundError( fnfErr);
	
	return newCopy;
  }

bool CopyAction::CouldBeCopy( const CatInfo& original,
										const CatInfo& copy )
  {
	if ( original.IsFile() )
		return copy.IsFile()
			 && original.CreationTime() == copy.CreationTime()
			 && original.ModificationTime() == copy.ModificationTime()
			 && original.File().Signature() == copy.File().Signature()
			 && original.File().Type() == copy.File().Type()
			 && original.File().Data().LogicalLength() == copy.File().Data().LogicalLength()
			 && original.File().Resources().LogicalLength() == copy.File().Resources().LogicalLength();
	 else
		return !copy.IsFile()
			 && original.CreationTime() == copy.CreationTime()
			 && original.ModificationTime() == copy.ModificationTime()
			 && original.Directory().FileCount() == copy.Directory().FileCount();
  }

void CopyAction::Undo()
  {
	AEDescriptor atDescriptor( event.Parameter( keyAEInsertHere ) );
	FileLocation at = HaveFinderResolve( atDescriptor );
	
	for ( uint32 i = selection.Length(); i > 0; i-- )
	  {
		AEDescriptor directObjectDescriptor( selection[i] );
		FileLocation directObject = HaveFinderResolve( directObjectDescriptor );
		FileLocation newCopy = FindCopy( directObject, at.AsDirectory() );

		DeleteRecursively( newCopy );
	  }

	HaveFinderSelect( selection );

	if ( sendUpdateEventsToFinder )
		HaveFinderUpdate( at );
  }

void CopyAction::Redo()
  {
	HaveFinderSelect( selection );
	
	AEEvent reply;
	ThrowOSError( AESend( &event,
								 &reply,
								 kAEWaitReply,
								 kAENormalPriority,
								 kAEDefaultTimeout,
								 AEEvent::IgnoreIncomingEvents(),
								 0 ) );	// filterProc

	HaveFinderSelect( selection );	
  }
