// DeleteAction.cp

#ifndef DeleteAction_h
#include "DeleteAction.h"
#endif
#ifndef FileLocation_h
#include "FileLocation.h"
#endif
#ifndef HaveFinderResolve_h
#include "HaveFinderResolve.h"
#endif
#ifndef AEAddress_h
#include "AEAddress.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef HaveFinderSelect_h
#include "HaveFinderSelect.h"
#endif
#ifndef ConfigureFinderUpdating_h
#include "ConfigureFinderUpdating.h"
#endif
#ifndef HaveFinderUpdate_h
#include "HaveFinderUpdate.h"
#endif

void DeleteAction::SetEvent( const AEList& newSelection,
									  const AEEvent& deleteEvent )
  {
	AEDescriptor selectionCopy( newSelection );
	selectionCopy.Swap( selection );

	AEDescriptor eventCopy( deleteEvent );
	eventCopy.Swap( event );
  }

void DeleteAction::Undo()
  {
	for ( uint32 i = selection.Length(); i > 0; i-- )
	  {
		AEDescriptor directObjectDescriptor( selection[i] );
		FileLocation source = HaveFinderResolveByParent( directObjectDescriptor );
		Directory trash = Directory::Find( kTrashFolderType, source.Volume() );
		FileLocation inTrash( trash, source.Name() );
		source.Up();
		ThrowOSError( FSpCatMove( &inTrash, &source ) );
	  }
	
	HaveFinderSelect( selection );
	
	if ( sendUpdateEventsToFinder )
	  {
		uint32 i = selection.Length();
		AEDescriptor directObjectDescriptor( selection[i] );
		FileLocation toUpdate = HaveFinderResolveByParent( directObjectDescriptor ).Parent();
		HaveFinderUpdate( toUpdate );
		Directory trash = Directory::Find( kTrashFolderType, toUpdate.Volume() );
		HaveFinderUpdate( FileLocation( trash ) );
		FileLocation updated = toUpdate;
		
		for ( i--; i > 0; i-- )
		  {
			AEDescriptor directObjectDescriptor( selection[i] );
			FileLocation toUpdate = HaveFinderResolveByParent( directObjectDescriptor ).Parent();
			
			if ( toUpdate == updated )
				continue;
			
			HaveFinderUpdate( toUpdate );
			
			if ( toUpdate.Volume() != updated.Volume() )
			  {
				Directory trash = Directory::Find( kTrashFolderType, toUpdate.Volume() );
				HaveFinderUpdate( FileLocation( trash ) );
			  }
				
			updated = toUpdate;
		  }
	  }
  }

void DeleteAction::Redo()
  {
	HaveFinderSelect( selection );
	
	AEEvent reply;
	ThrowOSError( AESend( &event,
								 &reply,
								 kAEWaitReply,
								 kAENormalPriority,
								 kAEDefaultTimeout,
								 AEEvent::IgnoreIncomingEvents(),
								 0 ) );	// filterProc
  }
