// DuplicateAction.cp

#ifndef DuplicateAction_h
#include "DuplicateAction.h"
#endif
#ifndef CatInfo_h
#include "CatInfo.h"
#endif
#ifndef HaveFinderUpdate_h
#include "HaveFinderUpdate.h"
#endif
#ifndef HaveFinderResolve_h
#include "HaveFinderResolve.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef HaveFinderSelect_h
#include "HaveFinderSelect.h"
#endif
#ifndef FinderNamingLoop_h
#include "FinderNamingLoop.h"
#endif
#ifndef ConfigureFinderUpdating_h
#include "ConfigureFinderUpdating.h"
#endif

void DuplicateAction::SetEvent( const AEList& newSelection,
										  const AEEvent& duplicateEvent )
  {
	AEDescriptor selectionCopy( newSelection );
	selectionCopy.Swap( selection );

	AEDescriptor eventCopy( duplicateEvent );
	eventCopy.Swap( event );
  }

FileLocation DuplicateAction::FindDuplicate( const FileLocation& original )
  {
	CatInfo originalInfo( original );
	
	FinderNamingLoop name( original.name, "\pcopy" );
	FileLocation newCopy = original.Sibling( *name );
	CatInfo newCopyInfo( newCopy );
	
	while ( !CouldBeDuplicate( originalInfo, newCopyInfo ) )
	  {
		name++;
		newCopy = original.Sibling( *name );
		newCopyInfo.Get( newCopy );
	  }
	
	for ( name++; true; name++ )
	  {
		FileLocation later = original.Sibling( *name );
		CatInfo laterInfo;
		
		if ( !laterInfo.TryToGet( later ) )
			break;
		
		if ( !CouldBeDuplicate( originalInfo, laterInfo ) )
			continue;
		
		newCopy = later;
		newCopyInfo = laterInfo;
	  }
	
	return newCopy;
  }

bool DuplicateAction::CouldBeDuplicate( const CatInfo& original,
													 const CatInfo& duplicate )
  {
	if ( original.IsFile() )
		return duplicate.IsFile()
			 && original.CreationTime() == duplicate.CreationTime()
			 && original.ModificationTime() == duplicate.ModificationTime()
			 && original.File().Signature() == duplicate.File().Signature()
			 && original.File().Type() == duplicate.File().Type()
			 && original.File().Data().LogicalLength() == duplicate.File().Data().LogicalLength()
			 && original.File().Resources().LogicalLength() == duplicate.File().Resources().LogicalLength();
	 else
		return !duplicate.IsFile()
			 && original.CreationTime() == duplicate.CreationTime()
			 && original.ModificationTime() == duplicate.ModificationTime()
			 && original.Directory().FileCount() == duplicate.Directory().FileCount();
  }

void DuplicateAction::Undo()
  {
	for ( uint32 i = selection.Length(); i > 0; i-- )
	  {
		AEDescriptor directObjectDescriptor( selection[i] );
		FileLocation directObject = HaveFinderResolve( directObjectDescriptor );
		FileLocation newItem = FindDuplicate( directObject );

		DeleteRecursively( newItem );
	  }

	HaveFinderSelect( selection );

	if ( sendUpdateEventsToFinder )
	  {
	  	uint32 i = selection.Length();
		AEDescriptor directObjectDescriptor( selection[i] );
		FileLocation toUpdate = HaveFinderResolve( directObjectDescriptor ).Parent();
		HaveFinderUpdate( toUpdate );
		FileLocation updated = toUpdate;
		
		for ( i--; i > 0; i-- )
		  {
			AEDescriptor directObjectDescriptor( selection[i] );
			FileLocation directObject = HaveFinderResolve( directObjectDescriptor );
			FileLocation newItem = FindDuplicate( directObject );
			FileLocation toUpdate = HaveFinderResolve( directObjectDescriptor ).Parent();
			
			if ( toUpdate == updated )
				continue;
				
			HaveFinderUpdate( toUpdate );
			updated = toUpdate;
		  }
	  }
  }

void DuplicateAction::Redo()
  {
	HaveFinderSelect( selection );
	
	AEEvent reply;
	ThrowOSError( AESend( &event,
								 &reply,
								 kAEWaitReply,
								 kAENormalPriority,
								 kAEDefaultTimeout,
								 AEEvent::IgnoreIncomingEvents(),
								 0 ) );	// filterProc
  }
