// MakeAliasAction.cp

#ifndef MakeAliasAction_h
#include "MakeAliasAction.h"
#endif
#ifndef FinderAliasNamingLoop_h
#include "FinderAliasNamingLoop.h"
#endif
#ifndef CatInfo_h
#include "CatInfo.h"
#endif
#ifndef HaveFinderUpdate_h
#include "HaveFinderUpdate.h"
#endif
#ifndef HaveFinderResolve_h
#include "HaveFinderResolve.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef HaveFinderSelect_h
#include "HaveFinderSelect.h"
#endif
#ifndef ConfigureFinderUpdating_h
#include "ConfigureFinderUpdating.h"
#endif

void MakeAliasAction::SetEvent( const AEList& newSelection,
										  const AEEvent& makeEvent )
  {
	AEDescriptor selectionCopy( newSelection );
	selectionCopy.Swap( selection );

	AEDescriptor eventCopy( makeEvent );
	eventCopy.Swap( event );
  }

FileLocation MakeAliasAction::FindNewAlias( const FileLocation& original,
														  Directory at )
  {
	FinderAliasNamingLoop name( original.name,
										 original.Parent() == at ? "\palias" : "\p" );
	FileLocation newAlias( at, *name );
	CatInfo newAliasInfo( newAlias );
	
	while ( !CouldBeNewAlias( original, newAliasInfo ) )
	  {
		name++;
		newAlias = FileLocation( at, *name );
		newAliasInfo.Get( newAlias );
	  }
	
	for ( name++; true; name++ )
	  {
		FileLocation later( at, *name );
		CatInfo laterInfo;
		
		if ( !laterInfo.TryToGet( later ) )
			break;
		
		if ( !CouldBeNewAlias( original, laterInfo ) )
			continue;
		
		if ( laterInfo.CreationTime() < newAliasInfo.CreationTime() )
			continue;
		
		newAlias = later;
		newAliasInfo = laterInfo;
	  }
	
	return newAlias;
  }

bool MakeAliasAction::CouldBeNewAlias( const FileLocation&,
													const CatInfo& info )
  {
	return info.IsFile() && info.File().IsAlias();
  }

void MakeAliasAction::Undo()
  {
	AEDescriptor atDescriptor( event.Parameter( keyAEInsertHere ) );
	FileLocation at = HaveFinderResolve( atDescriptor );
	
	for ( uint32 i = selection.Length(); i > 0; i-- )
	  {
		AEDescriptor toDescriptor( selection[i] );
		FileLocation to = HaveFinderResolve( toDescriptor );
		FileLocation newAlias = FindNewAlias( to, at.AsDirectory() );

		DeleteFile( newAlias );
	  }

	HaveFinderSelect( selection );

	if ( sendUpdateEventsToFinder )
		HaveFinderUpdate( at );
  }

void MakeAliasAction::Redo()
  {
	HaveFinderSelect( selection );
	
	AEEvent reply;
	ThrowOSError( AESend( &event,
								 &reply,
								 kAEWaitReply,
								 kAENormalPriority,
								 kAEDefaultTimeout,
								 AEEvent::IgnoreIncomingEvents(),
								 0 ) );	// filterProc

	AEDescriptor result( reply.Parameter( keyAEResult ) );
	HaveFinderSelect( result );
  }
