// MoveAction.cp

#ifndef MoveAction_h
#include "MoveAction.h"
#endif
#ifndef FileLocation_h
#include "FileLocation.h"
#endif
#ifndef HaveFinderResolve_h
#include "HaveFinderResolve.h"
#endif
#ifndef HaveFinderUpdate_h
#include "HaveFinderUpdate.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef HaveFinderSelect_h
#include "HaveFinderSelect.h"
#endif
#ifndef ConfigureFinderUpdating_h
#include "ConfigureFinderUpdating.h"
#endif

void MoveAction::SetEvent( const AEList& newSelection,
									const AEEvent& moveEvent )
  {
	AEDescriptor selectionCopy( newSelection );
	selectionCopy.Swap( selection );

	AEDescriptor eventCopy( moveEvent );
	eventCopy.Swap( event );
  }

void MoveAction::Undo()
  {
	AEDescriptor toDescriptor( event.Parameter( keyAEInsertHere ) );
	FileLocation toLocation = HaveFinderResolve( toDescriptor );
	Directory to = toLocation.AsDirectory();
	
	for ( uint32 i = selection.Length(); i > 0; i-- )
	  {
		AEDescriptor directObjectDescriptor( selection[i] );
		FileLocation source = HaveFinderResolveByParent( directObjectDescriptor );
		FileLocation destination( to, source.Name() );
		source.Up();
		
		ThrowOSError( FSpCatMove( &destination, &source ) );
	  }

	HaveFinderSelect( selection );

	if ( sendUpdateEventsToFinder )
	  {
		HaveFinderUpdate( toDescriptor );

		uint32 i = selection.Length();
		AEDescriptor directObjectDescriptor( selection[i] );
		FileLocation toUpdate = HaveFinderResolveByParent( directObjectDescriptor ).Parent();
		HaveFinderUpdate( toUpdate );
		FileLocation updated = toUpdate;
		
		for ( i--; i > 0; i-- )
		  {
			AEDescriptor directObjectDescriptor( selection[i] );
			FileLocation toUpdate = HaveFinderResolveByParent( directObjectDescriptor ).Parent();
			
			if ( toUpdate == updated )
				continue;
				
			HaveFinderUpdate( toUpdate );
			updated = toUpdate;
		  }
	  }
  }

void MoveAction::Redo()
  {
	HaveFinderSelect( selection );
	
	AEEvent reply;
	ThrowOSError( AESend( &event,
								 &reply,
								 kAEWaitReply,
								 kAENormalPriority,
								 kAEDefaultTimeout,
								 AEEvent::IgnoreIncomingEvents(),
								 0 ) );	// filterProc

	AEDescriptor result( reply.Parameter( keyAEResult ) );
	HaveFinderSelect( result );
  }
