// NewFolderAction.cp

#ifndef NewFolderAction_h
#include "NewFolderAction.h"
#endif
#ifndef CatInfo_h
#include "CatInfo.h"
#endif
#ifndef FinderNamingLoop_h
#include "FinderNamingLoop.h"
#endif
#ifndef AEEvent_h
#include "AEEvent.h"
#endif
#ifndef HaveFinderResolve_h
#include "HaveFinderResolve.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef HaveFinderUpdate_h
#include "HaveFinderUpdate.h"
#endif
#ifndef HaveFinderSelect_h
#include "HaveFinderSelect.h"
#endif
#ifndef ConfigureFinderUpdating_h
#include "ConfigureFinderUpdating.h"
#endif

void NewFolderAction::SetEvent( const AEEvent& event )
  {
	AEDescriptor copy( event );
	copy.Swap( makeFolderEvent );
  }

FileLocation NewFolderAction::FindNewFolder( Directory at )
  {
	FinderNamingLoop name( "\p", "\puntitled folder" );
	FileLocation newFolder( at, *name );
	CatInfo newFolderInfo( newFolder );
	
	while ( !CouldBeNewFolder( newFolderInfo ) )
	  {
		name++;
		newFolder = FileLocation( at, *name );
		newFolderInfo.Get( newFolder );
	  }
	
	for ( name++; true; name++ )
	  {
		FileLocation later( at, *name );
		CatInfo laterInfo;
		
		if ( !laterInfo.TryToGet( later ) )
			break;
		
		if ( !CouldBeNewFolder( laterInfo ) )
			continue;
		
		if ( laterInfo.CreationTime() < newFolderInfo.CreationTime() )
			continue;
		
		newFolder = later;
		newFolderInfo = laterInfo;
	  }
	
	return newFolder;
  }

bool NewFolderAction::CouldBeNewFolder( const CatInfo& info )
  {
	return info.IsDirectory()
	       && info.Directory().FileCount() == 0
	       && info.CreationTime() == info.ModificationTime();
  }

void NewFolderAction::Undo()
  {
	AEDescriptor atDescriptor( makeFolderEvent.Parameter( keyAEInsertHere ) );
	FileLocation at = HaveFinderResolve( atDescriptor );
	FileLocation newFolder = FindNewFolder( at.AsDirectory() );

	DeleteDirectory( newFolder );

	if ( sendUpdateEventsToFinder )
		HaveFinderUpdate( FileLocation( newFolder.Parent() ) );
  }

void NewFolderAction::Redo()
  {
	AEEvent reply;
	ThrowOSError( AESend( &makeFolderEvent,
								 &reply,
								 kAEWaitReply,
								 kAENormalPriority,
								 kAEDefaultTimeout,
								 AEEvent::IgnoreIncomingEvents(),
								 0 ) );	// filterProc

	AEDescriptor result( reply.Parameter( keyAEResult ) );
	HaveFinderSelect( result );
  }
