// UnfinderApplication.cp

#ifndef UnfinderApplication_h
#include "UnfinderApplication.h"
#endif
#ifndef Unfinder_h
#include "Unfinder.h"
#endif
#ifndef AEBoolean_h
#include "AEBoolean.h"
#endif
#ifndef AEEvent_h
#include "AEEvent.h"
#endif
#ifndef ResourceID_h
#include "ResourceID.h"
#endif
#ifndef MessageString_h
#include "MessageString.h"
#endif
#ifndef ProcessLoop_h
#include "ProcessLoop.h"
#endif
#ifndef ProcessInfo_h
#include "ProcessInfo.h"
#endif

UnfinderApplication::UnfinderApplication()
  : undo( kAECoreSuite,
  			 kAEUndo,
  			 *this,
  			 &UnfinderApplication::Undo ),
	 info( 'SCAM',
	 		 'info',
	 		 *this,
	 		 &UnfinderApplication::GetInfo )
  {
	Unfinder::The();
  }

UnfinderApplication& UnfinderApplication::The()
  {
  	static UnfinderApplication the;
	return the;
  }

void main()
  {
	UnfinderApplication::The().Run();
  }

bool UnfinderApplication::CanCreate() const
  {
	return true;
  }

void UnfinderApplication::Create()
  {
  }

void UnfinderApplication::Undo( const AEEvent&, AEEvent& )
  {
	Unfinder::The().UndoOrRedo();
  }

void UnfinderApplication::GetInfo( const AEEvent&, AEEvent& reply )
  {
	Undoing& unfinder( Unfinder::The() );

	ConstData actionName = unfinder.ActionName();
	bool enabled = unfinder.CanUndoOrRedo();
	
	const ResourceID nameResource( 128 );
		/* 
			1: Undo
		   2: Undo ^0
		   3: Can't Undo
		   4: Can't Undo ^0
		   5: Redo
		   6: Redo ^0
		   7: Can't Redo
		   8: Can't Redo ^0
		*/
	int16 nameIndex = 1 + ( unfinder.ActionIsDone()	? 0 : 4 )
							  + ( enabled						? 0 : 2 )
							  + ( actionName.IsEmpty()		? 0 : 1 );
	
	MessageString name( nameResource, nameIndex );
	if ( !actionName.IsEmpty() )
		name.Substitute( '0', actionName );

	reply.Parameter( 'able' ) = AEBoolean( enabled );
	reply.Parameter( 'name' ).Set( typeText, name.Text() );
  }
