// HaveFinderResolve.cp

#ifndef HaveFinderResolve_h
#include "HaveFinderResolve.h"
#endif
#ifndef AEEvent_h
#include "AEEvent.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef AEGetter_h
#include "AEGetter.h"
#endif
#ifndef FileLocation_h
#include "FileLocation.h"
#endif
#ifndef AEAddress_h
#include "AEAddress.h"
#endif
#ifndef AEStruct_h
#include "AEStruct.h"
#endif
#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif
#ifndef __AEREGISTRY__
#include <AERegistry.h>
#endif
#ifndef __FINDERREGISTRY__
#include <FinderRegistry.h>
#endif
#ifndef BufferMaster_h
#include "BufferMaster.h"
#endif

FileLocation HaveFinderResolve( const AEDescriptor& descriptor )
  {
	// The 8.6 Finder returns the wrong file spec when resolving "the desktop".
	  {
		AEStruct record( descriptor );
		
		if ( AEGetter<typeEnumerated>( record[ 'form' ] ) == cProperty
		     && AEGetter<typeType>( record[ 'seld' ] ) == pDesktop )
			return Directory::Find( kDesktopFolderType );
	  }
	
	static const AEAddress finder( FileSignature::Finder() );
	AEEvent getEvent( kAECoreSuite, kAEGetData, finder );
						
	getEvent.Parameter( keyDirectObject ) = descriptor;
	
	const DescType fileSpecType = typeFSS;
	const ConstData fileSpecTypeBlock( &fileSpecType, sizeof( fileSpecType ) );
	getEvent.Parameter( keyAERequestedType ).Set( typeType, fileSpecTypeBlock );
	
	AEEvent reply;
	ThrowOSError( AESend( &getEvent,
								 &reply,
								 kAEWaitReply,
								 kAENormalPriority,
								 kAEDefaultTimeout,
								 AEEvent::IgnoreIncomingEvents(),
								 0 ) );	// filterProc
		
	return AEGetter<typeFSS>( reply.Parameter( keyAEResult ) );
  }

FileLocation HaveFinderResolveByParent( const AEDescriptor& descriptor )
  {
	AEStruct record( descriptor );

	Assert( AEGetter<typeEnumerated>( record[ 'form' ] ) == 'name' );

	AEDescriptor parentDescriptor( record[ 'from' ] );
	FileLocation parent( HaveFinderResolve( parentDescriptor ) );
	
	Str63 name;
	BufferMaster buffer( Data( name + 1, sizeof(name) - 1 ) );
	record[ 'seld' ] >> buffer;
	name[0] = buffer.UsedLength();
	
	return parent.Child( name );
  }
