// UnfinderPatch.cp

#ifndef UnfinderPatch_h
#include "UnfinderPatch.h"
#endif
#ifndef OSError_h
#include "OSError.h"
#endif
#ifndef Patch_h
#include "Patch.h"
#endif
#ifndef AEAddress_h
#include "AEAddress.h"
#endif
#ifndef AEEvent_h
#include "AEEvent.h"
#endif
#ifndef AEGetter_h
#include "AEGetter.h"
#endif
#ifndef BufferMaster_h
#include "BufferMaster.h"
#endif
#ifndef ProcessInfo_h
#include "ProcessInfo.h"
#endif

bool InFinder();
void PrepareMenu();
long FilterResult( long );

void TryPrepareMenu();
long TryFilterResult( long );

pascal long MyMenuSelect( Point );
pascal long MyMenuKey( short );

static const short editMenuID = 0x0102;
static const short undoItem = 1;

static const long menuSelectProcInfo =
							kPascalStackBased
							| RESULT_SIZE(SIZE_CODE(sizeof(long)))
							| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(Point)));

static const long menuKeyProcInfo =
							kPascalStackBased
							| RESULT_SIZE(SIZE_CODE(sizeof(long)))
							| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(short)));

static UniversalProcPtr oldMenuSelect;
static UniversalProcPtr oldMenuKey;

bool InFinder()
  {
	return ProcessInfo( Process::front ).Signature() == FileSignature::Finder();
  }

void PrepareMenu()
  {
	MenuHandle editMenu = GetMenuHandle( editMenuID );
	if ( editMenu != 0 )
	  {
		static const AEEvent event( 'SCAM', 'info', AEAddress( FileSignature( 'SCAM' ) ) );

		AEEvent reply;
		ThrowOSError( AESend( &event,
									 &reply,
									 kAEWaitReply,
									 kAENormalPriority,
									 kAEDefaultTimeout,
									 AEEvent::IgnoreIncomingEvents(),
									 0 ) );	// filterProc
		
		bool enabled = AEGetter<typeBoolean>( reply.Parameter( 'able' ) );
		
		Str255 name;
		BufferMaster buffer( Data( name + 1, sizeof(name) - 1 ) );
		reply.Parameter( 'name' ) >> buffer;
		name[0] = buffer.UsedLength();
		
		SetMenuItemText( editMenu, undoItem, name );
		if ( enabled )
		  {
			EnableItem( editMenu, undoItem );
			EnableItem( editMenu, 0 );
		  }
		 else
			DisableItem( editMenu, undoItem );
	  }
  }

long FilterResult( long original )
  {
	if ( original != ((long( editMenuID ) << 16) | undoItem) )
		return original;

	static const AEEvent event( kAECoreSuite, kAEUndo, AEAddress( FileSignature( 'SCAM' ) ) );

	AEDescriptor reply;
	ThrowOSError( AESend( &event,
								 &reply,
								 kAENoReply,
								 kAENormalPriority,
								 kAEDefaultTimeout,
								 0,		// idleProc
								 0 ) );	// filterProc
	HiliteMenu( 0 );
	return 0;
  }

void TryPrepareMenu()
  {
	try
	  {
		PrepareMenu();
	  }
	 catch( ... )
	  {
		Assert(0);
		MenuHandle editMenu = GetMenuHandle( editMenuID );
		if ( editMenu != 0 )
		  {
			SetMenuItemText( editMenu, undoItem, "\pUndo" );
			DisableItem( editMenu, undoItem );
		  }
	  }
  }

int32 TryFilterResult( int32 original )
  {
	try
	  {
		return FilterResult( original );
	  }
	 catch( ... )
	  {
		Assert(0);
	  }
	
	return original;
  }

pascal long MyMenuSelect( Point start )
  {
	bool inFinder = InFinder();
	
	if ( inFinder )
		TryPrepareMenu();
	
	long result = CallUniversalProc( oldMenuSelect, menuSelectProcInfo, start );

	if ( inFinder )
		result = TryFilterResult( result );

	return result;
  }

pascal long MyMenuKey( short c )
  {
	bool inFinder = InFinder();
	
	if ( inFinder )
		TryPrepareMenu();

	long result = CallUniversalProc( oldMenuKey, menuKeyProcInfo, c );

	if ( inFinder )
		result = TryFilterResult( result );

	return result;
  }

pascal void InstallUnfinderPatch()
  {
	DetachResource( Get1Resource( 'INIT', 128 ) );

	#if GENERATINGCFM
		static RoutineDescriptor MyMenuSelect =
			BUILD_ROUTINE_DESCRIPTOR( menuSelectProcInfo, &::MyMenuSelect );

		static RoutineDescriptor MyMenuKey =
			BUILD_ROUTINE_DESCRIPTOR( menuKeyProcInfo, &::MyMenuKey );
	#endif

	// check that the signatures match the real traps:
		&::MyMenuSelect != &MenuSelect;
		&::MyMenuKey != &MenuKey;
	
	static PatchBase selectPatch( ToolTrap, _MenuSelect, menuSelectProcInfo, &MyMenuSelect );
	oldMenuSelect = selectPatch.Old();
	
	static PatchBase keyPatch( ToolTrap, _MenuKey, menuKeyProcInfo, &MyMenuKey );
	oldMenuKey = keyPatch.Old();
  }
